@@ -24,6 +24,7 @@
 #include "io/file_manager.hpp"
 #include "modes/world.hpp"
 #include "race/race_manager.hpp"
+#include "utils/vs.hpp"
 
 #include <pthread.h>
 #include <stdexcept>
@@ -295,6 +296,7 @@ void SFXManager::stopThread()
  */
 void* SFXManager::mainLoop(void *obj)
 {
+    VS::setThreadName("SFXManager");
     SFXManager *me = (SFXManager*)obj;
 
     pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
@@ -378,7 +380,10 @@ void* SFXManager::mainLoop(void *obj)
         case SFX_MUSIC_DEFAULT_VOLUME:
         {
             current->m_music_information->setDefaultVolume();
+            break;
         }
+        case SFX_CREATE_SOURCE:
+            current->m_sfx->init(); break;
         default: assert("Not yet supported.");
         }
         delete current;
@@ -409,6 +414,7 @@ void* SFXManager::mainLoop(void *obj)
         delete me->m_sfx_commands.getData().front();
         me->m_sfx_commands.getData().erase(me->m_sfx_commands.getData().begin());
     }
+    me->m_sfx_commands.unlock();
     return NULL;
 }   // mainLoop
 
