@@ -335,9 +335,15 @@ namespace UserConfigParams
     PARAM_PREFIX IntUserConfigParam          m_num_goals
             PARAM_DEFAULT(  IntUserConfigParam(3, "numgoals",
             &m_race_setup_group, "Default number of goals in soccer mode.") );
+    PARAM_PREFIX IntUserConfigParam          m_soccer_default_team
+            PARAM_DEFAULT(  IntUserConfigParam(0, "soccer-default-team",
+            &m_race_setup_group, "Default team in soccer mode for single player.") );
     PARAM_PREFIX IntUserConfigParam          m_soccer_time_limit
             PARAM_DEFAULT(  IntUserConfigParam(3, "soccer-time-limit",
-            &m_race_setup_group, "Limit in soccer time mode.") );
+            &m_race_setup_group, "Time limit in soccer mode.") );
+    PARAM_PREFIX BoolUserConfigParam         m_soccer_use_time_limit
+            PARAM_DEFAULT(  BoolUserConfigParam(false, "soccer-use-time-limit",
+            &m_race_setup_group, "Enable time limit in soccer mode.") );
     PARAM_PREFIX IntUserConfigParam          m_difficulty
             PARAM_DEFAULT(  IntUserConfigParam(0, "difficulty",
                             &m_race_setup_group,
@@ -509,17 +515,18 @@ namespace UserConfigParams
     /** If track debugging is enabled. */
     PARAM_PREFIX int m_track_debug PARAM_DEFAULT( false );
 
+    /** If random number of items is used in an arena. */
+    PARAM_PREFIX bool m_random_arena_item PARAM_DEFAULT( false );
+
     /** True if check structures should be debugged. */
     PARAM_PREFIX bool m_check_debug PARAM_DEFAULT( false );
 
-    /** Special debug camera: 0: normal camera;   1: being high over the kart;
-                              2: on ground level; 3: free first person camera; 
-                              4: straight behind kart */
-    PARAM_PREFIX int m_camera_debug PARAM_DEFAULT( false );
-
     /** True if physics debugging should be enabled. */
     PARAM_PREFIX bool m_physics_debug PARAM_DEFAULT( false );
 
+    /** True if fps should be printed each frame. */
+    PARAM_PREFIX bool m_fps_debug PARAM_DEFAULT(false);
+
     /** True if slipstream debugging is activated. */
     PARAM_PREFIX bool m_slipstream_debug  PARAM_DEFAULT( false );
 
@@ -584,9 +591,9 @@ namespace UserConfigParams
                             "stun.voxgratia.org",
                             "stun.xten.com") );
 
-    PARAM_PREFIX StringUserConfigParam m_packets_log_filename
-            PARAM_DEFAULT( StringUserConfigParam("packets_log.txt", "packets_log_filename",
-                                                 "Where to log received and sent packets.") );
+    PARAM_PREFIX BoolUserConfigParam m_log_packets
+            PARAM_DEFAULT( BoolUserConfigParam(false, "log-network-packets",
+                                                 "If all network packets should be logged") );
 
     // ---- Graphic Quality
     PARAM_PREFIX GroupUserConfigParam        m_graphics_quality
@@ -623,12 +630,12 @@ namespace UserConfigParams
             PARAM_DEFAULT(  BoolUserConfigParam(true, "weather_gfx",
                                      &m_graphics_quality, "Weather effects") );
     PARAM_PREFIX IntUserConfigParam        m_show_steering_animations
-            PARAM_DEFAULT(  IntUserConfigParam(ANIMS_ALL,
+            PARAM_DEFAULT(  IntUserConfigParam(ANIMS_PLAYERS_ONLY,
                             "steering_animations", &m_graphics_quality,
                 "Whether to display kart animations (0=disabled for all; "
                 "1=enabled for humans, disabled for AIs; 2=enabled for all") );
     PARAM_PREFIX IntUserConfigParam         m_anisotropic
-            PARAM_DEFAULT( IntUserConfigParam(8, "anisotropic",
+            PARAM_DEFAULT( IntUserConfigParam(4, "anisotropic",
                            &m_graphics_quality,
                            "Quality of anisotropic filtering (usual values include 2-4-8-16; 0 to disable)") );
     PARAM_PREFIX BoolUserConfigParam         m_trilinear
@@ -663,7 +670,7 @@ namespace UserConfigParams
                            "shadows_resoltion", &m_graphics_quality,
                            "Shadow resolution (0 = disabled") );
     PARAM_PREFIX BoolUserConfigParam          m_degraded_IBL
-        PARAM_DEFAULT(BoolUserConfigParam(false,
+        PARAM_DEFAULT(BoolUserConfigParam(true,
         "Degraded_IBL", &m_graphics_quality,
         "Disable specular IBL"));
 
