@@ -36,6 +36,7 @@ void CentralVideoSettings::init()
     hasBuffserStorage = false;
     hasDrawIndirect = false;
     hasComputeShaders = false;
+    hasArraysOfArrays = false;
     hasTextureStorage = false;
     hasTextureView = false;
     hasBindlessTexture = false;
@@ -45,11 +46,13 @@ void CentralVideoSettings::init()
     hasMultiDrawIndirect = false;
     hasTextureCompression = false;
     hasUBO = false;
+    hasExplicitAttribLocation = false;
     hasGS = false;
-    m_GI_has_artifact = false;
 
+    m_GI_has_artifact = false;
     m_need_rh_workaround = false;
     m_need_srgb_workaround = false;
+    m_need_srgb_visual_workaround = false;
 
     // Call to glGetIntegerv should not be made if --no-graphics is used
     if (!ProfileWorld::isNoGraphics())
@@ -102,6 +105,11 @@ void CentralVideoSettings::init()
             hasComputeShaders = true;
             Log::info("GLDriver", "ARB Compute Shader Present");
         }
+        if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_ARRAYS_OF_ARRAYS) &&
+            hasGLExtension("GL_ARB_arrays_of_arrays")) {
+            hasArraysOfArrays = true;
+            Log::info("GLDriver", "ARB Arrays of Arrays Present");
+        }
         if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_TEXTURE_STORAGE) &&
             hasGLExtension("GL_ARB_texture_storage")) {
             hasTextureStorage = true;
@@ -147,11 +155,24 @@ void CentralVideoSettings::init()
             hasUBO = true;
             Log::info("GLDriver", "ARB Uniform Buffer Object Present");
         }
-        if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_GEOMETRY_SHADER4) &&
+        if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_EXPLICIT_ATTRIB_LOCATION) &&
+            hasGLExtension("GL_ARB_explicit_attrib_location")) {
+            hasExplicitAttribLocation = true;
+            Log::info("GLDriver", "ARB Explicit Attrib Location Present");
+        }
+        #if defined(__linux__)
+        if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_GEOMETRY_SHADER) &&
+            (m_gl_major_version > 3 || (m_gl_major_version == 3 && m_gl_minor_version >= 2))) {
+            hasGS = true;
+            Log::info("GLDriver", "Geometry Shaders Present");
+        }
+        #else
+        if (!GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_GEOMETRY_SHADER) &&
             hasGLExtension("GL_ARB_geometry_shader4")) {
             hasGS = true;
             Log::info("GLDriver", "ARB Geometry Shader 4 Present");
         }
+        #endif
 
         // Only unset the high def textures if they are set as default. If the
         // user has enabled them (bit 1 set), then leave them enabled.
@@ -179,6 +200,16 @@ void CentralVideoSettings::init()
             // Bindless textures are all treated RGB even sRGB one
             m_need_srgb_workaround = true;
         }
+
+        // Check if visual is sRGB-capable
+        if (GraphicsRestrictions::isDisabled(GraphicsRestrictions::GR_FRAMEBUFFER_SRGB_CAPABLE) &&
+            m_glsl == true)
+        {
+            GLint param = GL_SRGB;
+            glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER, GL_BACK_LEFT,
+                              GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING, &param);
+            m_need_srgb_visual_workaround = (param != GL_SRGB);
+        }
     }
 }
 
@@ -207,7 +238,12 @@ bool CentralVideoSettings::needsRGBBindlessWorkaround() const
     return m_need_srgb_workaround;
 }
 
-bool CentralVideoSettings::isARBGeometryShader4Usable() const
+bool CentralVideoSettings::needsSRGBCapableVisualWorkaround() const
+{
+    return m_need_srgb_visual_workaround;
+}
+
+bool CentralVideoSettings::isARBGeometryShadersUsable() const
 {
     return hasGS;
 }
@@ -217,6 +253,11 @@ bool CentralVideoSettings::isARBUniformBufferObjectUsable() const
     return hasUBO;
 }
 
+bool CentralVideoSettings::isARBExplicitAttribLocationUsable() const
+{
+    return hasExplicitAttribLocation;
+}
+
 bool CentralVideoSettings::isEXTTextureCompressionS3TCUsable() const
 {
     return hasTextureCompression;
@@ -247,6 +288,11 @@ bool CentralVideoSettings::isARBComputeShaderUsable() const
     return hasComputeShaders;
 }
 
+bool CentralVideoSettings::isARBArraysOfArraysUsable() const
+{
+    return hasArraysOfArrays;
+}
+
 bool CentralVideoSettings::isARBTextureStorageUsable() const
 {
     return hasTextureStorage;
@@ -284,12 +330,12 @@ bool CentralVideoSettings::isARBMultiDrawIndirectUsable() const
 
 bool CentralVideoSettings::supportsShadows() const
 {
-    return isARBGeometryShader4Usable() && isARBUniformBufferObjectUsable();
+    return isARBGeometryShadersUsable() && isARBUniformBufferObjectUsable() && isARBExplicitAttribLocationUsable();
 }
 
 bool CentralVideoSettings::supportsGlobalIllumination() const
 {
-    return isARBGeometryShader4Usable() && isARBUniformBufferObjectUsable() && !m_GI_has_artifact;
+    return isARBGeometryShadersUsable() && isARBUniformBufferObjectUsable() && isARBExplicitAttribLocationUsable() && !m_GI_has_artifact;
 }
 
 bool CentralVideoSettings::supportsIndirectInstancingRendering() const
@@ -299,7 +345,7 @@ bool CentralVideoSettings::supportsIndirectInstancingRendering() const
 
 bool CentralVideoSettings::supportsComputeShadersFiltering() const
 {
-    return isARBBufferStorageUsable() && isARBImageLoadStoreUsable() && isARBComputeShaderUsable();
+    return isARBBufferStorageUsable() && isARBImageLoadStoreUsable() && isARBComputeShaderUsable() && isARBArraysOfArraysUsable();
 }
 
 bool CentralVideoSettings::supportsAsyncInstanceUpload() const
