@@ -41,13 +41,14 @@ namespace GraphicsRestrictions
 
         const char *m_names_of_restrictions[] = {
             "UniformBufferObject",
-            "GeometryShader4",
+            "GeometryShader",
             "DrawIndirect",
             "TextureView",
             "TextureStorage",
             "ImageLoadStore",
             "BaseInstance",
             "ComputeShader",
+            "ArraysOfArrays",
             "ShaderStorageBufferObject",
             "MultiDrawIndirect",
             "ShaderAtomicCounters",
@@ -55,10 +56,12 @@ namespace GraphicsRestrictions
             "BindlessTexture",
             "TextureCompressionS3TC",
             "AMDVertexShaderLayer",
+            "ExplicitAttribLocation",
             "DriverRecentEnough",
             "HighDefinitionTextures",
             "AdvancedPipeline",
             "FramebufferSRGBWorking",
+            "FramebufferSRGBCapable",
             "GI",
         };
     }   // namespace Private
