@@ -57,6 +57,7 @@
 #include "modes/profile_world.hpp"
 #include "modes/world.hpp"
 #include "physics/physics.hpp"
+#include "scriptengine/property_animator.hpp"
 #include "states_screens/dialogs/confirm_resolution_dialog.hpp"
 #include "states_screens/state_manager.hpp"
 #include "tracks/track_manager.hpp"
@@ -82,6 +83,7 @@ please use the included version."
 using namespace irr;
 
 #ifdef WIN32
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 #if defined(__linux__) && !defined(ANDROID)
@@ -129,7 +131,8 @@ IrrDriver::IrrDriver()
     m_lightviz = m_shadowviz = m_distortviz = m_rsm = m_rh = m_gi = false;
     m_boundingboxesviz = false;
     m_last_light_bucket_distance = 0;
-    memset(object_count, 0, sizeof(object_count));
+    memset(m_object_count, 0, sizeof(m_object_count));
+    memset(m_poly_count, 0, sizeof(m_poly_count));
 }   // IrrDriver
 
 // ----------------------------------------------------------------------------
@@ -186,12 +189,12 @@ STKRenderingPass IrrDriver::getPhase() const
 
 void IrrDriver::IncreaseObjectCount()
 {
-    object_count[m_phase]++;
+    m_object_count[m_phase]++;
 }
 
 void IrrDriver::IncreasePolyCount(unsigned Polys)
 {
-    poly_count[m_phase] += Polys;
+    m_poly_count[m_phase] += Polys;
 }
 
 core::array<video::IRenderTarget> &IrrDriver::getMainSetup()
@@ -344,6 +347,8 @@ void IrrDriver::createListOfVideoModes()
  */
 void IrrDriver::initDevice()
 {
+    SIrrlichtCreationParameters params;
+    
     // If --no-graphics option was used, the null device can still be used.
     if (!ProfileWorld::isNoGraphics())
     {
@@ -426,7 +431,6 @@ void IrrDriver::initDevice()
         m_device->drop();
         m_device  = NULL;
 
-        SIrrlichtCreationParameters params;
         params.ForceLegacyDevice = (UserConfigParams::m_force_legacy_device || 
             UserConfigParams::m_gamepad_visualisation);
 
@@ -447,7 +451,8 @@ void IrrDriver::initDevice()
             params.WindowSize    =
                 core::dimension2du(UserConfigParams::m_width,
                                    UserConfigParams::m_height);
-
+            params.HandleSRGB    = true;
+            
             /*
             switch ((int)UserConfigParams::m_antialiasing)
             {
@@ -505,6 +510,30 @@ void IrrDriver::initDevice()
     {
         Log::fatal("irr_driver", "Couldn't initialise irrlicht device. Quitting.\n");
     }
+    
+    CVS->init();
+                    
+    // This is the ugly hack for intel driver on linux, which doesn't
+    // use sRGB-capable visual, even if we request it. This causes
+    // the screen to be darker than expected. It affects mesa 10.6 and newer. 
+    // Though we are able to force to use the proper format on mesa side by 
+    // setting WithAlphaChannel parameter.
+    if (!ProfileWorld::isNoGraphics() && CVS->needsSRGBCapableVisualWorkaround())
+    {
+        Log::warn("irr_driver", "Created visual is not sRGB-capable. "
+                                "Re-creating device to workaround the issue.");
+        m_device->closeDevice();
+        m_device->drop();
+
+        params.WithAlphaChannel = true;
+        
+        m_device = createDeviceEx(params);
+        
+        if(!m_device)
+        {
+            Log::fatal("irr_driver", "Couldn't initialise irrlicht device. Quitting.\n");
+        }
+    }
 
     m_scene_manager = m_device->getSceneManager();
     m_gui_env       = m_device->getGUIEnvironment();
@@ -513,15 +542,13 @@ void IrrDriver::initDevice()
 
     m_actual_screen_size = m_video_driver->getCurrentRenderTargetSize();
 
-    CVS->init();
-
     m_spherical_harmonics = new SphericalHarmonics(m_scene_manager->getAmbientLight().toSColor());
 
     if (UserConfigParams::m_shadows_resolution != 0 &&
         (UserConfigParams::m_shadows_resolution < 512 ||
          UserConfigParams::m_shadows_resolution > 2048))
     {
-        Log::warn("IrrDriver", 
+        Log::warn("irr_driver", 
                "Invalid value for UserConfigParams::m_shadows_resolution : %i",
             (int)UserConfigParams::m_shadows_resolution);
         UserConfigParams::m_shadows_resolution = 0;
@@ -986,7 +1013,8 @@ scene::IMesh *IrrDriver::getMesh(const std::string &filename)
  *  \return Newly created skinned mesh. You should call drop() when you don't
  *          need it anymore.
  */
-scene::IAnimatedMesh *IrrDriver::copyAnimatedMesh(scene::IAnimatedMesh *orig)
+scene::IAnimatedMesh *IrrDriver::copyAnimatedMesh(scene::IAnimatedMesh *orig,
+                                                  video::E_RENDER_TYPE rt)
 {
     using namespace scene;
     CSkinnedMesh *mesh = dynamic_cast<CSkinnedMesh*>(orig);
@@ -995,7 +1023,10 @@ scene::IAnimatedMesh *IrrDriver::copyAnimatedMesh(scene::IAnimatedMesh *orig)
         Log::error("copyAnimatedMesh", "Given mesh was not a skinned mesh.");
         return NULL;
     }
-    return mesh->clone();
+
+    scene::IAnimatedMesh* out = mesh->clone();
+    out->setRenderType(rt);
+    return out;
 }   // copyAnimatedMesh
 
 // ----------------------------------------------------------------------------
@@ -1108,7 +1139,8 @@ scene::IMeshSceneNode *IrrDriver::addSphere(float radius,
     m.EmissiveColor   = color;
     m.BackfaceCulling = false;
     m.MaterialType    = video::EMT_SOLID;
-    m.setTexture(0, getUnicolorTexture(video::SColor(128, 255, 105, 180)));
+    //m.setTexture(0, getUnicolorTexture(video::SColor(128, 255, 105, 180)));
+    m.setTexture(0, getUnicolorTexture(color));
     m.setTexture(1, getUnicolorTexture(video::SColor(0, 0, 0, 0)));
 
     if (CVS->isGLSL())
@@ -1863,13 +1895,13 @@ void IrrDriver::displayFPS()
     {
         fpsString = _("FPS: %d/%d/%d  - PolyCount: %d Solid, "
                       "%d Shadows - LightDist : %d",
-                    min, fps, max, poly_count[SOLID_NORMAL_AND_DEPTH_PASS],
-                    poly_count[SHADOW_PASS], m_last_light_bucket_distance);
-        poly_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
-        poly_count[SHADOW_PASS] = 0;
-        object_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
-        object_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
-        object_count[TRANSPARENT_PASS] = 0;
+                    min, fps, max, m_poly_count[SOLID_NORMAL_AND_DEPTH_PASS],
+                    m_poly_count[SHADOW_PASS], m_last_light_bucket_distance);
+        m_poly_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
+        m_poly_count[SHADOW_PASS] = 0;
+        m_object_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
+        m_object_count[SOLID_NORMAL_AND_DEPTH_PASS] = 0;
+        m_object_count[TRANSPARENT_PASS] = 0;
     }
     else
         fpsString = _("FPS: %d/%d/%d - %d KTris", min, fps, max, (int)roundf(kilotris));
@@ -2072,7 +2104,7 @@ void IrrDriver::doScreenShot()
     video::IImage* image = m_video_driver->createScreenShot();
     if(!image)
     {
-        Log::error("IrrDriver", "Could not create screen shot.");
+        Log::error("irr_driver", "Could not create screen shot.");
         return;
     }
 
@@ -2152,27 +2184,33 @@ void IrrDriver::update(float dt)
 
     m_wind->update();
 
+    PropertyAnimator::get()->update(dt);
+
     World *world = World::getWorld();
 
-    if (GUIEngine::getCurrentScreen() != NULL &&
-        GUIEngine::getCurrentScreen()->needs3D() &&
-        world != NULL)
+    if (world)
     {
-        //printf("Screen that needs 3D\n");
-        //m_video_driver->beginScene(/*backBuffer clear*/true, /*zBuffer*/true,
-        //                           video::SColor(0,0,0,255));
-        //m_scene_manager->drawAll();
-
         if (CVS->isGLSL())
             renderGLSL(dt);
         else
             renderFixed(dt);
-
-        GUIEngine::render(dt);
-        //m_video_driver->endScene();
-        return;
+            
+        GUIEngine::Screen* current_screen = GUIEngine::getCurrentScreen();
+        if (current_screen != NULL && current_screen->needs3D())
+        {
+            GUIEngine::render(dt);
+        }
+        
+        if (world->getPhysics() != NULL)
+        {
+            IrrDebugDrawer* debug_drawer = world->getPhysics()->getDebugDrawer();
+            if (debug_drawer != NULL && debug_drawer->debugEnabled())
+            {
+                debug_drawer->beginNextFrame();
+            }
+        }
     }
-    else if (!world)
+    else
     {
         m_video_driver->beginScene(/*backBuffer clear*/ true, /*zBuffer*/ true,
                                    video::SColor(255,100,101,140));
@@ -2180,24 +2218,8 @@ void IrrDriver::update(float dt)
         GUIEngine::render(dt);
 
         m_video_driver->endScene();
-        return;
     }
-
-    if (CVS->isGLSL())
-        renderGLSL(dt);
-    else
-        renderFixed(dt);
-
-
-    if (world != NULL && world->getPhysics() != NULL)
-    {
-        IrrDebugDrawer* debug_drawer = world->getPhysics()->getDebugDrawer();
-        if (debug_drawer != NULL && debug_drawer->debugEnabled())
-        {
-            debug_drawer->beginNextFrame();
-        }
-    }
-
+    
     if (m_request_screenshot) doScreenShot();
 
     // Enable this next print statement to get render information printed
