@@ -27,6 +27,7 @@
 #include "graphics/shaders.hpp"
 #include "graphics/shared_gpu_objects.hpp"
 #include "graphics/stk_mesh_scene_node.hpp"
+#include "graphics/weather.hpp"
 #include "io/file_manager.hpp"
 #include "karts/abstract_kart.hpp"
 #include "karts/kart_model.hpp"
@@ -818,6 +819,24 @@ public:
     }   // render
 };   // SunLightShader
 
+// ============================================================================
+class LightningShader : public TextureShader<LightningShader, 1, 
+                                             core::vector3df>
+{
+public:
+    LightningShader()
+    {
+        loadProgram(OBJECT, GL_VERTEX_SHADER, "screenquad.vert",
+                            GL_FRAGMENT_SHADER, "lightning.frag");
+        assignUniforms("intensity");
+    }   // LightningShader
+    // ------------------------------------------------------------------------
+    void render(core::vector3df intensity)
+    {
+        drawFullScreenEffect(intensity);
+    }   // render
+};   // LightningShader
+
 // ============================================================================
 
 PostProcessing::PostProcessing(IVideoDriver* video_driver)
@@ -1230,8 +1249,6 @@ void PostProcessing::renderHorizontalBlur(const FrameBuffer &in_fbo,
 {
     assert(in_fbo.getWidth() == auxiliary.getWidth() &&
            in_fbo.getHeight() == auxiliary.getHeight());
-    float inv_width  = 1.0f / in_fbo.getWidth();
-    float inv_height = 1.0f / in_fbo.getHeight();
 
     auxiliary.bind();
     Gaussian6HBlurShader::getInstance()->render(in_fbo, in_fbo.getWidth(),
@@ -1250,8 +1267,6 @@ void PostProcessing::renderGaussian17TapBlur(const FrameBuffer &in_fbo,
            in_fbo.getHeight() == auxiliary.getHeight());
     if (CVS->supportsComputeShadersFiltering())
         glMemoryBarrier(GL_FRAMEBUFFER_BARRIER_BIT);
-    float inv_width = 1.0f / in_fbo.getWidth();
-    float inv_height = 1.0f / in_fbo.getHeight();
     {
         if (!CVS->supportsComputeShadersFiltering())
         {
@@ -1408,6 +1423,18 @@ void PostProcessing::applyMLAA()
     glDisable(GL_STENCIL_TEST);
 }   // applyMLAA
 
+// ----------------------------------------------------------------------------
+void PostProcessing::renderLightning(core::vector3df intensity)
+{
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_ONE, GL_ONE);
+    glBlendEquation(GL_FUNC_ADD);
+       
+    LightningShader::getInstance()->render(intensity);
+    
+    glDisable(GL_BLEND);
+}
+
 // ----------------------------------------------------------------------------
 /** Render the post-processed scene */
 FrameBuffer *PostProcessing::render(scene::ICameraSceneNode * const camnode,
@@ -1608,6 +1635,22 @@ FrameBuffer *PostProcessing::render(scene::ICameraSceneNode * const camnode,
         }
         PROFILER_POP_CPU_MARKER();
     }
+    
+    // Handle lightning rendering
+    {
+        PROFILER_PUSH_CPU_MARKER("- Lightning", 0xFF, 0x00, 0x00);
+        ScopedGPUTimer Timer(irr_driver->getGPUTimer(Q_LIGHTNING));
+        if (World::getWorld() != NULL)
+        {
+            Weather* m_weather = World::getWorld()->getWeather();
+            
+            if (m_weather != NULL && m_weather->shouldLightning())
+            {
+                renderLightning(m_weather->getIntensity());
+            }
+        }
+        PROFILER_POP_CPU_MARKER();
+    }
 
     // Workaround a bug with srgb fbo on sandy bridge windows
     if (!CVS->isARBUniformBufferObjectUsable())
