@@ -25,8 +25,8 @@
 #include "graphics/material_manager.hpp"
 #include "graphics/stk_mesh_scene_node.hpp"
 #include "io/file_manager.hpp"
-#include "karts/controller/controller.hpp"
 #include "karts/abstract_kart.hpp"
+#include "karts/controller/controller.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/max_speed.hpp"
 #include "modes/world.hpp"
@@ -67,11 +67,9 @@ SlipStream::SlipStream(AbstractKart* kart) : MovingTexture(0, 0), m_kart(kart)
     setTextureMatrix(&(m_node->getMaterial(0).getTextureMatrix(0)));
     m_slipstream_time      = 0.0f;
 
-    float length = m_kart->getKartProperties()->getSlipstreamLength() *
-                   m_kart->getPlayerDifficulty()->getSlipstreamLength();
+    float length = m_kart->getKartProperties()->getSlipstreamLength();
     float kw     = m_kart->getKartWidth();
-    float ew     = m_kart->getKartProperties()->getSlipstreamWidth() *
-                   m_kart->getPlayerDifficulty()->getSlipstreamWidth();
+    float ew     = m_kart->getKartProperties()->getSlipstreamWidth();
     float kl     = m_kart->getKartLength();
 
     Vec3 p[4];
@@ -314,8 +312,7 @@ void SlipStream::setIntensity(float f, const AbstractKart *kart)
 bool SlipStream::isSlipstreamReady() const
 {
     return m_slipstream_time>
-        m_kart->getKartProperties()->getSlipstreamCollectTime() *
-        m_kart->getPlayerDifficulty()->getSlipstreamCollectTime();
+        m_kart->getKartProperties()->getSlipstreamCollectTime();
 }   // isSlipstreamReady
 
 //-----------------------------------------------------------------------------
@@ -329,16 +326,12 @@ void SlipStream::updateSlipstreamPower()
     if(m_slipstream_mode==SS_USE)
     {
         setIntensity(2.0f, NULL);
-        const KartProperties *kp=m_kart->getKartProperties();
+        const KartProperties *kp = m_kart->getKartProperties();
         m_kart->increaseMaxSpeed(MaxSpeed::MS_INCREASE_SLIPSTREAM,
-                                kp->getSlipstreamMaxSpeedIncrease() *
-                                m_kart->getPlayerDifficulty()->getSlipstreamMaxSpeedIncrease(),
-                                kp->getSlipstreamAddPower() *
-                                m_kart->getPlayerDifficulty()->getSlipstreamAddPower(),
-                                kp->getSlipstreamDuration() *
-                                m_kart->getPlayerDifficulty()->getSlipstreamDuration(),
-                                kp->getSlipstreamFadeOutTime() *
-                                m_kart->getPlayerDifficulty()->getSlipstreamFadeOutTime());
+                                kp->getSlipstreamMaxSpeedIncrease(),
+                                kp->getSlipstreamAddPower(),
+                                kp->getSlipstreamDuration(),
+                                kp->getSlipstreamFadeOutTime());
     }
 }   // upateSlipstreamPower
 
@@ -368,6 +361,8 @@ void SlipStream::setDebugColor(const video::SColor &color)
  */
 void SlipStream::update(float dt)
 {
+    const KartProperties *kp = m_kart->getKartProperties();
+
     // Low level AIs should not do any slipstreaming.
     if(m_kart->getController()->disableSlipstreamBonus())
         return;
@@ -394,8 +389,7 @@ void SlipStream::update(float dt)
     // not moving. This is useful for debugging the graphics of SS-ing.
 //#define DISPLAY_SLIPSTREAM_WITH_0_SPEED_FOR_DEBUGGING
 #ifndef DISPLAY_SLIPSTREAM_WITH_0_SPEED_FOR_DEBUGGING
-    if(m_kart->getSpeed()<m_kart->getKartProperties()->getSlipstreamMinSpeed() *
-                          m_kart->getPlayerDifficulty()->getSlipstreamMinSpeed())
+    if(m_kart->getSpeed() < kp->getSlipstreamMinSpeed())
     {
         setIntensity(0, NULL);
         m_slipstream_mode = SS_NONE;
@@ -436,12 +430,10 @@ void SlipStream::update(float dt)
         // entirely sure if this makes sense, but it makes it easier to
         // give karts different slipstream properties.
 #ifndef DISPLAY_SLIPSTREAM_WITH_0_SPEED_FOR_DEBUGGING
-        if(m_target_kart->getSpeed() <
-            m_kart->getKartProperties()->getSlipstreamMinSpeed() *
-            m_kart->getPlayerDifficulty()->getSlipstreamMinSpeed())
+        if (m_target_kart->getSpeed() < kp->getSlipstreamMinSpeed())
         {
             if(UserConfigParams::m_slipstream_debug &&
-                m_kart->getController()->isPlayerController())
+                m_kart->getController()->isLocalPlayerController())
                 m_target_kart->getSlipstream()
                               ->setDebugColor(video::SColor(255, 0, 0, 0));
 
@@ -452,14 +444,13 @@ void SlipStream::update(float dt)
         // slipstream length+0.5*kart_length()+0.5*target_kart_length
         // away from the other kart
         Vec3 delta = m_kart->getXYZ() - m_target_kart->getXYZ();
-        float l    = m_kart->getKartProperties()->getSlipstreamLength() *
-                     m_kart->getPlayerDifficulty()->getSlipstreamLength()
+        float l    = kp->getSlipstreamLength()
                    + 0.5f*( m_target_kart->getKartLength()
                            +m_kart->getKartLength()        );
         if(delta.length2_2d() > l*l)
         {
             if(UserConfigParams::m_slipstream_debug &&
-                m_kart->getController()->isPlayerController())
+                m_kart->getController()->isLocalPlayerController())
                 m_target_kart->getSlipstream()
                              ->setDebugColor(video::SColor(255, 0, 0, 128));
             continue;
@@ -472,7 +463,7 @@ void SlipStream::update(float dt)
             break;
         }
         if(UserConfigParams::m_slipstream_debug &&
-            m_kart->getController()->isPlayerController())
+            m_kart->getController()->isLocalPlayerController())
             m_target_kart->getSlipstream()
                          ->setDebugColor(video::SColor(255, 0, 0, 255));
     }   // for i < num_karts
@@ -480,7 +471,7 @@ void SlipStream::update(float dt)
     if(!is_sstreaming)
     {
         if(UserConfigParams::m_slipstream_debug &&
-            m_kart->getController()->isPlayerController())
+            m_kart->getController()->isLocalPlayerController())
             m_target_kart->getSlipstream()
                          ->setDebugColor(video::SColor(255, 255, 0, 0));
 
@@ -493,9 +484,7 @@ void SlipStream::update(float dt)
             {
                 m_slipstream_mode = SS_USE;
                 m_kart->handleZipper();
-                m_slipstream_time =
-                    m_kart->getKartProperties()->getSlipstreamCollectTime() *
-                    m_kart->getPlayerDifficulty()->getSlipstreamCollectTime();
+                m_slipstream_time = kp->getSlipstreamCollectTime();
                 return;
             }
         }
@@ -506,7 +495,7 @@ void SlipStream::update(float dt)
     }   // if !is_sstreaming
 
     if(UserConfigParams::m_slipstream_debug &&
-        m_kart->getController()->isPlayerController())
+        m_kart->getController()->isLocalPlayerController())
         m_target_kart->getSlipstream()->setDebugColor(video::SColor(255, 0, 255, 0));
     // Accumulate slipstream credits now
     m_slipstream_time = m_slipstream_mode==SS_NONE ? dt
@@ -516,8 +505,7 @@ void SlipStream::update(float dt)
     setIntensity(m_slipstream_time, m_target_kart);
 
     m_slipstream_mode = SS_COLLECT;
-    if(m_slipstream_time>m_kart->getKartProperties()->getSlipstreamCollectTime() *
-                         m_kart->getPlayerDifficulty()->getSlipstreamCollectTime())
+    if (m_slipstream_time > kp->getSlipstreamCollectTime())
     {
         setIntensity(1.0f, m_target_kart);
     }
