@@ -688,6 +688,8 @@ namespace GUIEngine
     {
         IGUIEnvironment* g_env;
         Skin* g_skin = NULL;
+        FTEnvironment* g_ft_env = NULL;
+        GlyphPageCreator* g_gp_creator = NULL;
         ScalableFont *g_font;
         ScalableFont *g_outline_font;
         ScalableFont *g_large_font;
@@ -952,6 +954,11 @@ namespace GUIEngine
         //if (g_skin != NULL) delete g_skin;
         g_skin = NULL;
 
+        delete g_ft_env;
+        g_ft_env = NULL;
+        delete g_gp_creator;
+        g_gp_creator = NULL;
+
         for (unsigned int i=0; i<g_loaded_screens.size(); i++)
         {
             g_loaded_screens[i].unload();
@@ -982,6 +989,17 @@ namespace GUIEngine
         // kill everything along the device
     }   // cleanUp
 
+    // -----------------------------------------------------------------------
+     void cleanHollowCopyFont()
+    {
+        g_small_font->drop();
+        g_small_font = NULL;
+        g_large_font->drop();
+        g_large_font = NULL;
+        g_outline_font->drop();
+        g_outline_font = NULL;
+    }   // cleanHollowCopyFont
+
     // -----------------------------------------------------------------------
 
     /**
@@ -1009,6 +1027,9 @@ namespace GUIEngine
             g_focus_for_player[n] = NULL;
         }
 
+        g_ft_env = new FTEnvironment();
+        g_gp_creator = new GlyphPageCreator();
+
         /*
          To make the g_font a little bit nicer, we load an external g_font
          and set it as the new default g_font in the g_skin.
@@ -1042,45 +1063,25 @@ namespace GUIEngine
             }
         }
 
-        // font size is resolution-dependent.
-        // normal text will range from 0.8, in 640x* resolutions (won't scale
-        // below that) to 1.0, in 1024x* resolutions, and linearly up
-        // normal text will range from 0.2, in 640x* resolutions (won't scale
-        // below that) to 0.4, in 1024x* resolutions, and linearly up
-        const int screen_width = irr_driver->getFrameSize().Width;
-        const int screen_height = irr_driver->getFrameSize().Height;
-        float scale = std::max(0, screen_width - 640)/564.0f;
-
-        // attempt to compensate for small screens
-        if (screen_width < 1200) scale = std::max(0, screen_width - 640) / 750.0f;
-        if (screen_width < 900 || screen_height < 700) scale = std::min(scale, 0.05f);
-
-        Log::info("GUIEngine", "scale: %f", scale);
-
-        float normal_text_scale = 0.7f + 0.2f*scale;
-        float title_text_scale = 0.2f + 0.2f*scale;
-
-        ScalableFont* sfont =
-            new ScalableFont(g_env,
-                            file_manager->getAssetChecked(FileManager::FONT,
-                                                          "StkFont.xml",true) );
-        sfont->setScale(normal_text_scale);
-        sfont->setKerningHeight(-5);
-        g_font = sfont;
-
-        ScalableFont* digit_font =
-            new ScalableFont(g_env,
-                             file_manager->getAssetChecked(FileManager::FONT,
-                                                           "BigDigitFont.xml",true));
-        digit_font->lazyLoadTexture(0); // make sure the texture is loaded for this one
+        ScalableFont* digit_font =new ScalableFont(g_env,T_DIGIT);
         digit_font->setMonospaceDigits(true);
         g_digit_font = digit_font;
 
+        ScalableFont* sfont2 =new ScalableFont(g_env,T_BOLD);
+        sfont2->setKerningWidth(0);
+        // Because the fallback font is much smaller than the title font:
+        sfont2->m_fallback_font_scale = 2.0f;
+        sfont2->m_fallback_kerning_width = 0;
+
+        ScalableFont* sfont =new ScalableFont(g_env,T_NORMAL);
+        sfont->setKerningHeight(0);
+        sfont->setScale(1);
+        g_font = sfont;
         Private::font_height = g_font->getDimension( L"X" ).Height;
 
         ScalableFont* sfont_larger = sfont->getHollowCopy();
-        sfont_larger->setScale(normal_text_scale*1.4f);
-        sfont_larger->setKerningHeight(-5);
+        sfont_larger->setScale(1.4f);
+        sfont_larger->setKerningHeight(0);
         g_large_font = sfont_larger;
 
         g_outline_font = sfont->getHollowCopy();
@@ -1089,25 +1090,15 @@ namespace GUIEngine
         Private::large_font_height = g_large_font->getDimension( L"X" ).Height;
 
         ScalableFont* sfont_smaller = sfont->getHollowCopy();
-        sfont_smaller->setScale(normal_text_scale*0.8f);
-        sfont_smaller->setKerningHeight(-5);
+        sfont_smaller->setScale(0.8f);
+        sfont_smaller->setKerningHeight(0);
         g_small_font = sfont_smaller;
 
         Private::small_font_height =
             g_small_font->getDimension( L"X" ).Height;
 
-
-        ScalableFont* sfont2 =
-            new ScalableFont(g_env,
-                             file_manager->getAssetChecked(FileManager::FONT,
-                                                           "title_font.xml",
-                                                           true)             );
         sfont2->m_fallback_font = sfont;
-        // Because the fallback font is much smaller than the title font:
-        sfont2->m_fallback_font_scale = 4.0f;
-        sfont2->m_fallback_kerning_width = 15;
-        sfont2->setScale(title_text_scale);
-        sfont2->setKerningWidth(-18);
+        sfont2->setScale(1);
         sfont2->m_black_border = true;
         g_title_font = sfont2;
         Private::title_font_height =
@@ -1125,6 +1116,32 @@ namespace GUIEngine
         g_device->getVideoDriver()->endScene();
     }   // init
 
+    // -----------------------------------------------------------------------
+    void reloadHollowCopyFont(irr::gui::ScalableFont* sfont)
+    {
+        //Base on the init function above
+        sfont->setScale(1);
+        sfont->setKerningHeight(0);
+        Private::font_height = sfont->getDimension( L"X" ).Height;
+
+        ScalableFont* sfont_larger = sfont->getHollowCopy();
+        sfont_larger->setScale(1.4f);
+        sfont_larger->setKerningHeight(0);
+        g_large_font = sfont_larger;
+
+        g_outline_font = sfont->getHollowCopy();
+        g_outline_font->m_black_border = true;
+
+        Private::large_font_height = g_large_font->getDimension( L"X" ).Height;
+
+        ScalableFont* sfont_smaller = sfont->getHollowCopy();
+        sfont_smaller->setScale(0.8f);
+        sfont_smaller->setKerningHeight(0);
+        g_small_font = sfont_smaller;
+
+        Private::small_font_height =  g_small_font->getDimension( L"X" ).Height;
+    }   // reloadHollowCopyFont
+
     // -----------------------------------------------------------------------
     void reloadSkin()
     {
