@@ -24,6 +24,7 @@
 
 #include "config/user_config.hpp"
 #include "graphics/2dutils.hpp"
+#include "graphics/central_settings.hpp"
 #include "guiengine/engine.hpp"
 #include "guiengine/modaldialog.hpp"
 #include "guiengine/scalable_font.hpp"
@@ -1958,14 +1959,31 @@ void Skin::process3DPane(IGUIElement *element, const core::recti &rect,
     else if (type == WTYPE_MODEL_VIEW)
     {
         ModelViewWidget* mvw = dynamic_cast<ModelViewWidget*>(widget);
-        FrameBuffer* fb = mvw->getFrameBuffer();
-        if (fb != NULL && fb->getRTT().size() > 0)
+        
+        if (CVS->isGLSL())
         {
-            draw2DImageFromRTT(fb->getRTT()[0], 512, 512,
-                rect, core::rect<s32>(0, 0, 512, 512), NULL, SColor(255, 255, 255, 255), true);
+            FrameBuffer* fb = mvw->getFrameBuffer();
+            if (fb != NULL && fb->getRTT().size() > 0)
+            {
+                glEnable(GL_FRAMEBUFFER_SRGB);
+                draw2DImageFromRTT(fb->getRTT()[0], 512, 512,
+                                   rect, core::rect<s32>(0, 0, 512, 512), NULL, 
+                                   SColor(255, 255, 255, 255), true);
+                glDisable(GL_FRAMEBUFFER_SRGB);
+            }
+        }
+        else
+        {
+            video::ITexture* texture = mvw->getTexture();
+            if (texture != NULL && texture->getSize().Width > 0 
+                                && texture->getSize().Height > 0)
+            {
+                draw2DImage(texture, rect, core::rect<s32>(0, 0, 512, 512),
+                            NULL, NULL, true);
+            }
         }
     }
-    else if (type == WTYPE_ICON_BUTTON || type == WTYPE_MODEL_VIEW)
+    else if (type == WTYPE_ICON_BUTTON)
     {
         drawIconButton(rect, widget, pressed, focused);
     }
