@@ -13,6 +13,7 @@
 #include "Keycodes.h"
 
 #include "graphics/2dutils.hpp"
+#include "utils/string_utils.hpp"
 #include "utils/translation.hpp"
 #include "utils/time.hpp"
 
@@ -23,8 +24,18 @@
     double click/ctrl click: word select + drag to select whole words, triple click to select line
     optional? dragging selected text
     numerical
+    correct the mark position in RTL text, currently you can identify by highlight the text
 */
 
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+#define UTF16_IS_SURROGATE_LO(c)	(((c) & 0xFC00) == 0xDC00)
+#define UTF16_IS_SURROGATE_HI(c)	(((c) & 0xFC00) == 0xD800)
+
+namespace irr
+{
+    void updateICPos(void* hWnd, s32 x, s32 y, s32 height);
+}
+#endif
 
 StkTime::TimeType getTime()
 {
@@ -43,7 +54,11 @@ CGUIEditBox::CGUIEditBox(const wchar_t* text, bool border,
     PasswordChar(L'*'), HAlign(EGUIA_UPPERLEFT), VAlign(EGUIA_CENTER),
     CurrentTextRect(0,0,1,1), FrameRect(rectangle)
 {
-    m_rtl = is_rtl;
+    //m_rtl = is_rtl;
+    m_rtl = false;
+    // FIXME quick hack to enable mark movement with keyboard and mouse for rtl language,
+    // don't know why it's disabled in the first place, because STK fail
+    // to input unicode characters before?
 
     #ifdef _DEBUG
     setDebugName("CGUIEditBox");
@@ -228,6 +243,12 @@ bool CGUIEditBox::OnEvent(const SEvent& event)
                 }
             }
             break;
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+        case EET_IMPUT_METHOD_EVENT:
+            if (processIMEEvent(event))
+                return true;
+            break;
+#endif
         case EET_KEY_INPUT_EVENT:
             if (processKey(event))
                 return true;
@@ -279,9 +300,13 @@ bool CGUIEditBox::processKey(const SEvent& event)
                 const s32 realmbgn = MarkBegin < MarkEnd ? MarkBegin : MarkEnd;
                 const s32 realmend = MarkBegin < MarkEnd ? MarkEnd : MarkBegin;
 
-                core::stringc s;
+                core::stringw s;
                 s = Text.subString(realmbgn, realmend - realmbgn).c_str();
+#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
                 Operator->copyToClipboard(s.c_str());
+#else
+                Operator->copyToClipboard(StringUtils::wideToUtf8(s).c_str());
+#endif
             }
             break;
         case KEY_KEY_X:
@@ -292,9 +317,13 @@ bool CGUIEditBox::processKey(const SEvent& event)
                 const s32 realmend = MarkBegin < MarkEnd ? MarkEnd : MarkBegin;
 
                 // copy
-                core::stringc sc;
+                core::stringw sc;
                 sc = Text.subString(realmbgn, realmend - realmbgn).c_str();
+#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
                 Operator->copyToClipboard(sc.c_str());
+#else
+                Operator->copyToClipboard(StringUtils::wideToUtf8(sc).c_str());
+#endif
 
                 if (isEnabled())
                 {
@@ -323,20 +352,32 @@ bool CGUIEditBox::processKey(const SEvent& event)
                 const s32 realmend = MarkBegin < MarkEnd ? MarkEnd : MarkBegin;
 
                 // add new character
+#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+                const wchar_t* p = Operator->getTextFromClipboard();
+#else
                 const c8* p = Operator->getTextFromClipboard();
+#endif
                 if (p)
                 {
                     if (MarkBegin == MarkEnd)
                     {
                         // insert text
                         core::stringw s = Text.subString(0, CursorPos);
+#ifndef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+                        s.append(StringUtils::utf8ToWide(p));
+#else
                         s.append(p);
+#endif
                         s.append( Text.subString(CursorPos, Text.size()-CursorPos) );
 
                         if (!Max || s.size()<=Max) // thx to Fish FH for fix
                         {
                             Text = s;
+#ifndef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+                            s = StringUtils::utf8ToWide(p);
+#else
                             s = p;
+#endif
                             CursorPos += s.size();
                         }
                     }
@@ -345,13 +386,22 @@ bool CGUIEditBox::processKey(const SEvent& event)
                         // replace text
 
                         core::stringw s = Text.subString(0, realmbgn);
+
+#ifndef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+                        s.append(StringUtils::utf8ToWide(p));
+#else
                         s.append(p);
+#endif
                         s.append( Text.subString(realmend, Text.size()-realmend) );
 
                         if (!Max || s.size()<=Max)  // thx to Fish FH for fix
                         {
                             Text = s;
+#ifndef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+                            s = StringUtils::utf8ToWide(p);
+#else
                             s = p;
+#endif
                             CursorPos = realmbgn + s.size();
                         }
                     }
@@ -733,10 +783,102 @@ bool CGUIEditBox::processKey(const SEvent& event)
 
     calculateScrollPos();
 
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+    switch(event.KeyInput.Key)
+    {
+    // If cursor points the surrogate low, send KEY_LEFT event.
+    case KEY_UP:
+    case KEY_DOWN:
+        if (MultiLine || (WordWrap && BrokenText.size() > 1) )
+        {
+            if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+            {
+                SEvent leftEvent;
+                leftEvent = event;
+                leftEvent.KeyInput.Key = KEY_LEFT;
+                Environment->postEventFromUser(leftEvent);
+            }
+        }
+        break;
+    // If cursor points the surrogate low, send a same event.
+    case KEY_LEFT:
+    case KEY_RIGHT:
+    case KEY_DELETE:
+        if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+            Environment->postEventFromUser(event);
+        break;
+    // If cursor points front of the surrogate high, send a same event.
+    case KEY_BACK:
+        if (CursorPos > 0)
+        {
+            if (UTF16_IS_SURROGATE_HI(Text[CursorPos-1]))
+                Environment->postEventFromUser(event);
+        }
+        break;
+    default:
+        break;
+    }
+#endif
     return true;
 }
 
 
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+bool CGUIEditBox::processIMEEvent(const SEvent& event)
+{
+    switch(event.InputMethodEvent.Event)
+    {
+    case EIME_CHAR_INPUT:
+        inputChar(event.InputMethodEvent.Char);
+        return true;
+    case EIME_CHANGE_POS:
+    {
+        core::position2di pos = calculateICPos();
+
+        IGUIFont* font = OverrideFont;
+        IGUISkin* skin = Environment->getSkin();
+
+        if (!OverrideFont)
+            font = skin->getFont();
+
+        irr::updateICPos(event.InputMethodEvent.Handle, pos.X,pos.Y, font->getDimension(L"|").Height);
+
+        return true;
+    }
+    default:
+        break;
+    }
+
+    return false;
+}
+
+//! calculate the position of input composition window
+core::position2di CGUIEditBox::calculateICPos()
+{
+    core::position2di pos;
+    IGUIFont* font = OverrideFont;
+    IGUISkin* skin = Environment->getSkin();
+    if (!OverrideFont)
+        font = skin->getFont();
+
+    //drop the text that clipping on the right side
+    if (WordWrap | MultiLine)
+    {
+        // todo : It looks like a heavy drinker. Strange!!
+        pos.X = CurrentTextRect.LowerRightCorner.X - font->getDimension(Text.subString(CursorPos, BrokenTextPositions[getLineFromPos(CursorPos)] + BrokenText[getLineFromPos(CursorPos)].size() - CursorPos).c_str()).Width;
+        pos.Y = CurrentTextRect.UpperLeftCorner.Y + font->getDimension(L"|").Height + (Border ? 3 : 0) - ((MultiLine | WordWrap) ? 3 : 0);
+    }
+    else
+    {
+        pos.X = CurrentTextRect.LowerRightCorner.X - font->getDimension(Text.subString(CursorPos, Text.size() - CursorPos).c_str()).Width;
+        pos.Y = AbsoluteRect.getCenter().Y + (Border ? 3 : 0); //bug? The text is always drawn in the height of the center. SetTextAlignment() doesn't influence.
+    }
+
+    return pos;
+}
+
+
+#endif
 //! draws the element and its children
 void CGUIEditBox::draw()
 {
@@ -848,20 +990,12 @@ void CGUIEditBox::draw()
                     startPos = ml ? BrokenTextPositions[i] : 0;
                 }
 
-
-                if (m_rtl)
-                {
-                    font->draw(translations->fribidize(txtLine->c_str()), CurrentTextRect,
-                               OverrideColorEnabled ? OverrideColor : skin->getColor(EGDC_BUTTON_TEXT),
-                               false, true, &localClipRect);
-                }
-                else
-                {
-                    // draw normal text
-                    font->draw(txtLine->c_str(), CurrentTextRect,
-                        OverrideColorEnabled ? OverrideColor : skin->getColor(EGDC_BUTTON_TEXT),
-                        false, true, &localClipRect);
-                }
+                font->draw(translations->fribidize(txtLine->c_str()), CurrentTextRect,
+                           OverrideColorEnabled ? OverrideColor : skin->getColor(EGDC_BUTTON_TEXT),
+                           false, true, &localClipRect);
+                // draw with fribidize no matter what language, because in fribidize function,
+                // it will return the input pointer if (this->isRTLLanguage()) from Translations::isRTLText
+                // is false
 
                 // draw mark and marked text
                 if (focus && MarkBegin != MarkEnd && i >= hlineStart && i < hlineStart + hlineCount)
@@ -1027,6 +1161,13 @@ bool CGUIEditBox::processMouse(const SEvent& event)
         if (Environment->hasFocus(this) && !m_rtl)
         {
             CursorPos = getCursorPos(event.MouseInput.X, event.MouseInput.Y);
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+            if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+            {
+                if (CursorPos > 0)
+                    --CursorPos;
+            }
+#endif
             if (MouseMarking)
             {
                 setTextMarkers( MarkBegin, CursorPos );
@@ -1041,6 +1182,13 @@ bool CGUIEditBox::processMouse(const SEvent& event)
             if (MouseMarking)
             {
                 CursorPos = getCursorPos(event.MouseInput.X, event.MouseInput.Y);
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+                if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+                {
+                    if (CursorPos > 0)
+                        --CursorPos;
+                }
+#endif
                 setTextMarkers( MarkBegin, CursorPos );
                 calculateScrollPos();
                 return true;
@@ -1053,6 +1201,13 @@ bool CGUIEditBox::processMouse(const SEvent& event)
             BlinkStartTime = getTime();
             MouseMarking = true;
             CursorPos = getCursorPos(event.MouseInput.X, event.MouseInput.Y);
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+            if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+            {
+                if (CursorPos > 0)
+                    --CursorPos;
+            }
+#endif
             setTextMarkers(CursorPos, CursorPos );
             calculateScrollPos();
             return true;
@@ -1069,6 +1224,13 @@ bool CGUIEditBox::processMouse(const SEvent& event)
                 // move cursor
                 CursorPos = getCursorPos(event.MouseInput.X, event.MouseInput.Y);
 
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
+                if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
+                {
+                    if (CursorPos > 0)
+                        --CursorPos;
+                }
+#endif
                 s32 newMarkBegin = MarkBegin;
                 if (!MouseMarking)
                     newMarkBegin = CursorPos;
