@@ -370,6 +370,9 @@ void RibbonWidget::add()
     m_element->setTabOrder(id);
     m_element->setTabGroup(false);
     updateSelection();
+
+    if (!m_is_visible)
+        setVisible(false);
 }   // add
 
 // ----------------------------------------------------------------------------
@@ -690,7 +693,8 @@ EventPropagation RibbonWidget::transmitEvent(Widget* w,
     // bring focus back to enclosing ribbon widget
     this->setFocusForPlayer( playerID );
 
-    if (m_selection[playerID] != -1)
+    if (m_selection[playerID] > -1 && 
+        m_selection[playerID] < (int)(m_active_children.size()))
     {
         if (m_active_children[m_selection[playerID]].m_deactivated)
         {
