@@ -145,8 +145,8 @@ void SpinnerWidget::add()
     else
     {
         rect<s32> subsize_label = rect<s32>(m_h, 0, m_w - m_h, m_h);
-        const wchar_t *text = stringw(m_value).c_str();
-        IGUIStaticText* label = GUIEngine::getGUIEnv()->addStaticText(text, subsize_label,
+        stringw text = stringw(m_value);
+        IGUIStaticText* label = GUIEngine::getGUIEnv()->addStaticText(text.c_str(), subsize_label,
                                                                       false /* border */, true /* word wrap */,
                                                                       btn, getNewNoFocusID());
         m_children[1].m_element = label;
