@@ -39,8 +39,9 @@
 #include "replay/replay_recorder.hpp"
 #include "states_screens/kart_selection.hpp"
 #include "states_screens/main_menu_screen.hpp"
-#include "states_screens/options_screen_input2.hpp"
+#include "states_screens/options_screen_device.hpp"
 #include "states_screens/state_manager.hpp"
+#include "utils/debug.hpp"
 #include "utils/string_utils.hpp"
 
 #include <ISceneManager.h>
@@ -109,6 +110,14 @@ void InputManager::handleStaticAction(int key, int value)
     }
 
 
+    if (world != NULL && UserConfigParams::m_artist_debug_mode && 
+        control_is_pressed && value > 0)
+    {
+        if (Debug::handleStaticAction(key))
+            return;
+    }
+
+    // TODO: move debug shortcuts to Debug::handleStaticAction
     switch (key)
     {
 #ifdef DEBUG
@@ -131,6 +140,7 @@ void InputManager::handleStaticAction(int key, int value)
             break;
         }
 #endif
+
         case KEY_CONTROL:
         case KEY_RCONTROL:
         case KEY_LCONTROL:
@@ -164,89 +174,97 @@ void InputManager::handleStaticAction(int key, int value)
         // Moving the first person camera
         case KEY_KEY_W:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.Z = value ? cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.Z = value ? cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         case KEY_KEY_S:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.Z = value ? -cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && UserConfigParams::m_artist_debug_mode && 
+                Camera::isFPS()                                   )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.Z = value ? -cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         case KEY_KEY_D:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.X = value ? -cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && !UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                     )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.X = value ? -cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         case KEY_KEY_A:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.X = value ? cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.X = value ? cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         case KEY_KEY_R:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.Y = value ? cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.Y = value ? cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         case KEY_KEY_F:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *cam = Camera::getActiveCamera();
-            core::vector3df vel(cam->getLinearVelocity());
-            vel.Y = value ? -cam->getMaximumVelocity() : 0;
-            cam->setLinearVelocity(vel);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *cam = Camera::getActiveCamera();
+                core::vector3df vel(cam->getLinearVelocity());
+                vel.Y = value ? -cam->getMaximumVelocity() : 0;
+                cam->setLinearVelocity(vel);
+            }
             break;
         }
         // Rotating the first person camera
         case KEY_KEY_Q:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *active_cam = Camera::getActiveCamera();
-            active_cam->setAngularVelocity(value ?
-                UserConfigParams::m_fpscam_max_angular_velocity : 0.0f);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *active_cam = Camera::getActiveCamera();
+                active_cam->setAngularVelocity(value ?
+                    UserConfigParams::m_fpscam_max_angular_velocity : 0.0f);
+            }
             break;
         }
         case KEY_KEY_E:
         {
-            if (!world || !UserConfigParams::m_artist_debug_mode ||
-                UserConfigParams::m_camera_debug != 3) break;
-
-            Camera *active_cam = Camera::getActiveCamera();
-            active_cam->setAngularVelocity(value ?
-                -UserConfigParams::m_fpscam_max_angular_velocity : 0);
+            if (world && UserConfigParams::m_artist_debug_mode &&
+                Camera::isFPS()                                   )
+            {
+                Camera *active_cam = Camera::getActiveCamera();
+                active_cam->setAngularVelocity(value ?
+                    -UserConfigParams::m_fpscam_max_angular_velocity : 0);
+            }
             break;
         }
 
@@ -357,8 +375,8 @@ void InputManager::handleStaticAction(int key, int value)
         case KEY_F10:
             if(world && value)
             {
-                if(control_is_pressed && ReplayRecorder::get())
-                    ReplayRecorder::get()->Save();
+                if(control_is_pressed)
+                    ReplayRecorder::get()->save();
                 else
                     history->Save();
             }
@@ -425,7 +443,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
             sensed_input.m_device_id      = deviceID;
             sensed_input.m_button_id      = button;
             sensed_input.m_character      = deviceID;
-            OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+            OptionsScreenDevice::getInstance()->gotSensedInput(sensed_input);
             return;
         }
         break;
@@ -437,7 +455,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
             sensed_input.m_device_id      = deviceID;
             sensed_input.m_button_id      = button;
             sensed_input.m_character      = deviceID;
-            OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+            OptionsScreenDevice::getInstance()->gotSensedInput(sensed_input);
             return;
         }
         break;
@@ -472,7 +490,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
                                                            : Input::AD_NEGATIVE;
                 sensed_input.m_axis_range     = Input::AR_FULL;
                 sensed_input.m_character      = deviceID;
-                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+                OptionsScreenDevice::getInstance()->gotSensedInput(sensed_input);
 
             }
             else m_sensed_input_high_gamepad.insert(input_id);
@@ -494,7 +512,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
                 sensed_input.m_axis_range     = id_was_zero ? Input::AR_HALF
                                                             : Input::AR_FULL;
                 sensed_input.m_character      = deviceID;
-                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+                OptionsScreenDevice::getInstance()->gotSensedInput(sensed_input);
             }
             else if( inverse_id_was_high )
             {
@@ -510,7 +528,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
                 sensed_input.m_axis_range     = id_was_zero ? Input::AR_HALF
                                                             : Input::AR_FULL;
                 sensed_input.m_character      = deviceID;
-                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+                OptionsScreenDevice::getInstance()->gotSensedInput(sensed_input);
             }
             else
             {
@@ -700,7 +718,8 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
 
         // ... when in-game
         if (StateManager::get()->getGameState() == GUIEngine::GAME &&
-             !GUIEngine::ModalDialog::isADialogActive()                  )
+             !GUIEngine::ModalDialog::isADialogActive()            &&
+             !race_manager->isWatchingReplay() )
         {
             if (player == NULL)
             {
@@ -721,6 +740,12 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
             Controller* controller = pk->getController();
             if (controller != NULL) controller->action(action, abs(value));
         }
+        else if (race_manager->isWatchingReplay())
+        {
+            // Get the first ghost kart
+            World::getWorld()->getKart(0)
+                ->getController()->action(action, abs(value));
+        }
         // ... when in menus
         else
         {
@@ -921,20 +946,8 @@ EventPropagation InputManager::input(const SEvent& event)
             // escape is a little special
             if (key == KEY_ESCAPE)
             {
-                // Exit from first person view if activated
-                if (!GUIEngine::ModalDialog::isADialogActive() &&
-                    StateManager::get()->getGameState() == GUIEngine::GAME &&
-                    UserConfigParams::m_camera_debug == 3)
-                {
-                    UserConfigParams::m_camera_debug = 0;
-                    irr_driver->getDevice()->getCursorControl()->setVisible(true);
-                    return EVENT_BLOCK;
-                }
-                else
-                {
-                    StateManager::get()->escapePressed();
-                    return EVENT_BLOCK;
-                }
+                StateManager::get()->escapePressed();
+                return EVENT_BLOCK;
             }
             // 'backspace' in a text control must never be mapped, since user
             // can be in a text area trying to erase text (and if it's mapped
@@ -996,7 +1009,7 @@ EventPropagation InputManager::input(const SEvent& event)
 
         if (type == EMIE_MOUSE_MOVED)
         {
-            if (UserConfigParams::m_camera_debug == 3)
+            if (Camera::isFPS())
             {
                 Camera *cam = Camera::getActiveCamera();
                 // Center of the screen
@@ -1051,7 +1064,7 @@ EventPropagation InputManager::input(const SEvent& event)
         }
         else if (type == EMIE_MOUSE_WHEEL)
         {
-            if (UserConfigParams::m_camera_debug == 3)
+            if (Camera::isFPS())
             {
                 // Use scrolling to change the maximum speed
                 // Only test if it's more or less than 0 as it seems to be not
