@@ -52,6 +52,7 @@ namespace irr {
 #  include <dirent.h>
 #  include <unistd.h>
 #else
+#  define WIN32_LEAN_AND_MEAN
 #  include <direct.h>
 #  include <windows.h>
 #  include <stdio.h>
@@ -64,6 +65,7 @@ namespace irr {
 
 
 std::vector<std::string> FileManager::m_root_dirs;
+std::string              FileManager::m_stdout_filename = "stdout.log";
 
 #ifdef __APPLE__
 // dynamic data path detection onmac
@@ -115,18 +117,19 @@ FileManager::FileManager()
 {
     m_subdir_name.resize(ASSET_COUNT);
     m_subdir_name[CHALLENGE  ] = "challenges";
-    m_subdir_name[FONT       ] = "fonts";
     m_subdir_name[GFX        ] = "gfx";
     m_subdir_name[GRANDPRIX  ] = "grandprix";
     m_subdir_name[GUI        ] = "gui";
     m_subdir_name[LIBRARY    ] = "library";
     m_subdir_name[MODEL      ] = "models";
     m_subdir_name[MUSIC      ] = "music";
+    m_subdir_name[REPLAY     ] = "replay";
     m_subdir_name[SCRIPT     ] = "tracks";
     m_subdir_name[SFX        ] = "sfx";
     m_subdir_name[SKIN       ] = "skins";
     m_subdir_name[SHADER     ] = "shaders";
     m_subdir_name[TEXTURE    ] = "textures";
+    m_subdir_name[TTF        ] = "ttf";
     m_subdir_name[TRANSLATION] = "po";
 #ifdef __APPLE__
     // irrLicht's createDevice method has a nasty habit of messing the CWD.
@@ -211,6 +214,7 @@ FileManager::FileManager()
     checkAndCreateConfigDir();
     checkAndCreateAddonsDir();
     checkAndCreateScreenshotDir();
+    checkAndCreateReplayDir();
     checkAndCreateCachedTexturesDir();
     checkAndCreateGPDir();
 
@@ -639,6 +643,14 @@ std::string FileManager::getScreenshotDir() const
     return m_screenshot_dir;
 }   // getScreenshotDir
 
+//-----------------------------------------------------------------------------
+/** Returns the directory in which replay file should be stored.
+ */
+std::string FileManager::getReplayDir() const
+{
+    return m_replay_dir;
+}   // getReplayDir
+
 //-----------------------------------------------------------------------------
 /** Returns the directory in which resized textures should be cached.
 */
@@ -908,6 +920,32 @@ void FileManager::checkAndCreateScreenshotDir()
 
 }   // checkAndCreateScreenshotDir
 
+// ----------------------------------------------------------------------------
+/** Creates the directories for replay recorded. This will set m_replay_dir
+ *  with the appropriate path.
+ */
+void FileManager::checkAndCreateReplayDir()
+{
+#if defined(WIN32) || defined(__CYGWIN__)
+    m_replay_dir = m_user_config_dir + "replay/";
+#elif defined(__APPLE__)
+    m_replay_dir  = getenv("HOME");
+    m_replay_dir += "/Library/Application Support/SuperTuxKart/replay/";
+#else
+    m_replay_dir = checkAndCreateLinuxDir("XDG_DATA_HOME", "supertuxkart",
+                                          ".local/share", ".supertuxkart");
+    m_replay_dir += "replay/";
+#endif
+
+    if(!checkAndCreateDirectory(m_replay_dir))
+    {
+        Log::error("FileManager", "Can not create replay directory '%s', "
+                   "falling back to '.'.", m_replay_dir.c_str());
+        m_replay_dir = ".";
+    }
+
+}   // checkAndCreateReplayDir
+
 // ----------------------------------------------------------------------------
 /** Creates the directories for cached textures. This will set
 *  m_cached_textures_dir with the appropriate path.
@@ -1044,6 +1082,15 @@ std::string FileManager::checkAndCreateLinuxDir(const char *env_name,
 }   // checkAndCreateLinuxDir
 #endif
 
+//-----------------------------------------------------------------------------
+/** Sets the name for the stdout log file.
+ *  \param filename Filename to use (relative to the user config dir).
+ */
+void FileManager::setStdoutName(const std::string& filename)
+{
+    m_stdout_filename = filename;
+}   // setStdoutName
+
 //-----------------------------------------------------------------------------
 /** Redirects output to go into files in the user's config directory
  *  instead of to the console. It keeps backup copies of previous stdout files
@@ -1053,7 +1100,7 @@ void FileManager::redirectOutput()
 {
     // Do a simple log rotate: stdout.log.2 becomes stdout.log.3 etc
     const int NUM_BACKUPS=3;
-    std::string logoutfile = getUserConfigFile("stdout.log");
+    std::string logoutfile = getUserConfigFile(m_stdout_filename);
     for(int i=NUM_BACKUPS; i>1; i--)
     {
         std::ostringstream out_old;
