@@ -48,8 +48,8 @@ public:
      *  The last entry ASSET_COUNT specifies the number of entries. */
     enum AssetType {ASSET_MIN,
                     CHALLENGE=ASSET_MIN,
-                    FONT, GFX, GRANDPRIX, GUI, LIBRARY, MODEL, MUSIC,
-                    SCRIPT, SFX, SHADER, SKIN, TEXTURE, 
+                    GFX, GRANDPRIX, GUI, LIBRARY, MODEL, MUSIC, REPLAY,
+                    SCRIPT, SFX, SHADER, SKIN, TEXTURE, TTF,
                     TRANSLATION, ASSET_MAX = TRANSLATION,
                     ASSET_COUNT};
 private:
@@ -69,9 +69,15 @@ private:
     /** The list of all root directories. */
     static std::vector<std::string> m_root_dirs;
 
+    /** Name of stdout file. */
+    static std::string m_stdout_filename;
+
     /** Directory to store screenshots in. */
     std::string       m_screenshot_dir;
 
+    /** Directory to store replays in. */
+    std::string       m_replay_dir;
+
     /** Directory where resized textures are cached. */
     std::string       m_cached_textures_dir;
 
@@ -94,6 +100,7 @@ private:
     bool              isDirectory(const std::string &path) const;
     void              checkAndCreateAddonsDir();
     void              checkAndCreateScreenshotDir();
+    void              checkAndCreateReplayDir();
     void              checkAndCreateCachedTexturesDir();
     void              checkAndCreateGPDir();
     void              discoverPaths();
@@ -109,11 +116,13 @@ public:
                      ~FileManager();
     void              init();
     static void       addRootDirs(const std::string &roots);
+    static void       setStdoutName(const std::string &name);
     io::IXMLReader   *createXMLReader(const std::string &filename);
     XMLNode          *createXMLTree(const std::string &filename);
     XMLNode          *createXMLTreeFromString(const std::string & content);
 
     std::string       getScreenshotDir() const;
+    std::string       getReplayDir() const;
     std::string       getCachedTexturesDir() const;
     std::string       getGPDir() const;
     std::string       getTextureCacheLocation(const std::string& filename);
@@ -129,7 +138,13 @@ public:
                                 bool abort_on_error=false) const;
     std::string getAsset(AssetType type, const std::string &name) const;
     std::string getAsset(const std::string &name) const;
-
+    // ------------------------------------------------------------------------
+    /** Returns the directory of an asset. */
+    std::string getAssetDirectory(AssetType type) const
+    {
+        return m_subdir_name[type];
+    }
+    // ------------------------------------------------------------------------
     std::string searchMusic(const std::string& file_name) const;
     std::string searchTexture(const std::string& fname) const;
     std::string getUserConfigFile(const std::string& fname) const;
@@ -142,6 +157,9 @@ public:
         return fileExists(std::string(prefix) + path);
     }
     // ------------------------------------------------------------------------
+    /** Returns the name of the stdout file for log messages. */
+    static const std::string& getStdoutName() { return m_stdout_filename; }
+    // ------------------------------------------------------------------------
     void        listFiles        (std::set<std::string>& result,
                                   const std::string& dir,
                                   bool make_full_path=false) const;
