@@ -225,9 +225,7 @@ void Attachment::clear()
 */
 void Attachment::hitBanana(Item *item, int new_attachment)
 {
-    const StateManager::ActivePlayer *const ap = m_kart->getController()
-                                                       ->getPlayer();
-    if(ap && ap->getConstProfile()==PlayerManager::getCurrentPlayer())
+    if(m_kart->getController()->canGetAchievements())
         PlayerManager::increaseAchievement(AchievementInfo::ACHIEVE_BANANA,
                                            "banana",1                      );
     //Bubble gum shield effect:
@@ -249,14 +247,15 @@ void Attachment::hitBanana(Item *item, int new_attachment)
         return;
     }
 
+    const KartProperties *kp = m_kart->getKartProperties();
     switch(getType())   // If there already is an attachment, make it worse :)
     {
     case ATTACH_BOMB:
         {
         add_a_new_item = false;
         HitEffect *he = new Explosion(m_kart->getXYZ(), "explosion", "explosion_bomb.xml");
-        if(m_kart->getController()->isPlayerController())
-            he->setPlayerKartHit();
+        if(m_kart->getController()->isLocalPlayerController())
+            he->setLocalPlayerKartHit();
         projectile_manager->addHitEffect(he);
         ExplosionAnimation::create(m_kart);
         clear();
@@ -266,20 +265,18 @@ void Attachment::hitBanana(Item *item, int new_attachment)
         // default time. This is necessary to avoid that a kart lands on the
         // same banana again once the explosion animation is finished, giving
         // the kart the same penalty twice.
-        float f = std::max(item->getDisableTime(),
-                         m_kart->getKartProperties()->getExplosionTime() *
-                         m_kart->getPlayerDifficulty()->getExplosionTime() + 2.0f);
+        float f = std::max(item->getDisableTime(), kp->getExplosionDuration() + 2.0f);
         item->setDisableTime(f);
         break;
         }
     case ATTACH_ANVIL:
         // if the kart already has an anvil, attach a new anvil,
         // and increase the overall time
-        new_attachment = 2;
+        new_attachment = 1;
         leftover_time  = m_time_left;
         break;
     case ATTACH_PARACHUTE:
-        new_attachment = 2;
+        new_attachment = 0;
         leftover_time  = m_time_left;
         break;
     default:
@@ -288,7 +285,12 @@ void Attachment::hitBanana(Item *item, int new_attachment)
         m_kart->playCustomSFX(SFXManager::CUSTOM_ATTACH);
 
         if(new_attachment==-1)
-            new_attachment = m_random.get(3);
+        {
+            if(race_manager->getMinorMode() == RaceManager::MINOR_MODE_TIME_TRIAL)
+                new_attachment = m_random.get(2);
+            else
+                new_attachment = m_random.get(3);
+        }
     }   // switch
 
     if (add_a_new_item)
@@ -296,7 +298,7 @@ void Attachment::hitBanana(Item *item, int new_attachment)
         switch (new_attachment)
         {
         case 0:
-            set( ATTACH_PARACHUTE,stk_config->m_parachute_time+leftover_time);
+            set(ATTACH_PARACHUTE, kp->getParachuteDuration() + leftover_time);
             m_initial_speed = m_kart->getSpeed();
 
             // if going very slowly or backwards,
@@ -304,20 +306,20 @@ void Attachment::hitBanana(Item *item, int new_attachment)
             if(m_initial_speed <= 1.5) m_initial_speed = 1.5;
             break ;
         case 1:
-            set( ATTACH_BOMB, stk_config->m_bomb_time+leftover_time);
-
-            // if ( m_kart == m_kart[0] )
-            //   sound -> playSfx ( SOUND_SHOOMF ) ;
-            break ;
-        case 2:
-            set( ATTACH_ANVIL, stk_config->m_anvil_time+leftover_time);
+            set(ATTACH_ANVIL, kp->getAnvilDuration() + leftover_time);
             // if ( m_kart == m_kart[0] )
             //   sound -> playSfx ( SOUND_SHOOMF ) ;
             // Reduce speed once (see description above), all other changes are
             // handled in Kart::updatePhysics
-            m_kart->adjustSpeed(stk_config->m_anvil_speed_factor);
+            m_kart->adjustSpeed(kp->getAnvilSpeedFactor());
             m_kart->updateWeight();
             break ;
+        case 2:
+            set( ATTACH_BOMB, stk_config->m_bomb_time+leftover_time);
+
+            // if ( m_kart == m_kart[0] )
+            //   sound -> playSfx ( SOUND_SHOOMF ) ;
+            break ;
         }   // switch
     }
 }   // hitBanana
@@ -420,12 +422,14 @@ void Attachment::update(float dt)
         // This percentage is based on the ratio of
         // initial_speed / initial_max_speed
 
-        float f = m_initial_speed / stk_config->m_parachute_max_speed;
+        const KartProperties *kp = m_kart->getKartProperties();
+
+        float f = m_initial_speed / kp->getParachuteMaxSpeed();
         if (f > 1.0f) f = 1.0f;   // cap fraction
         if (m_kart->getSpeed() <= m_initial_speed *
-                                 (stk_config->m_parachute_lbound_fraction +
-                                  f * (  stk_config->m_parachute_ubound_fraction
-                                       - stk_config->m_parachute_lbound_fraction)))
+                                 (kp->getParachuteLboundFraction() +
+                                  f * (kp->getParachuteUboundFraction()
+                                     - kp->getParachuteLboundFraction())))
         {
             m_time_left = -1;
         }
@@ -454,8 +458,8 @@ void Attachment::update(float dt)
         if(m_time_left<=0.0)
         {
             HitEffect *he = new Explosion(m_kart->getXYZ(), "explosion", "explosion_bomb.xml");
-            if(m_kart->getController()->isPlayerController())
-                he->setPlayerKartHit();
+            if(m_kart->getController()->isLocalPlayerController())
+                he->setLocalPlayerKartHit();
             projectile_manager->addHitEffect(he);
             ExplosionAnimation::create(m_kart);
 
@@ -507,6 +511,12 @@ void Attachment::update(float dt)
         clear();
 }   // update
 
+// ----------------------------------------------------------------------------
+float Attachment::weightAdjust() const
+{
+    return m_type == ATTACH_ANVIL ? m_kart->getKartProperties()->getAnvilWeight() : 0.0f;
+}
+
 // ----------------------------------------------------------------------------
 /** Inform any eventual plugin when an animation is done. */
 void Attachment::OnAnimationEnd(scene::IAnimatedMeshSceneNode* node)
