@@ -165,7 +165,7 @@ bool Plunger::hit(AbstractKart *kart, PhysicalObject *obj)
         if(kart)
         {
             kart->blockViewWithPlunger();
-            if (kart->getController()->isPlayerController())
+            if (kart->getController()->isLocalPlayerController())
                 SFXManager::get()->quickSound("plunger");
         }
 
@@ -176,8 +176,7 @@ bool Plunger::hit(AbstractKart *kart, PhysicalObject *obj)
     }
     else
     {
-        m_keep_alive = m_owner->getKartProperties()->getRubberBandDuration() *
-                       m_owner->getPlayerDifficulty()->getRubberBandDuration();
+        m_keep_alive = m_owner->getKartProperties()->getPlungerBandDuration();
 
         // Make this object invisible by placing it faaar down. Not that if this
         // objects is simply removed from the scene graph, it might be auto-deleted
