@@ -19,10 +19,12 @@
 #ifndef HEADER_ABSTRACT_KART_HPP
 #define HEADER_ABSTRACT_KART_HPP
 
+#include <EMaterialTypes.h>
+#include <memory>
+
 #include "items/powerup_manager.hpp"
 #include "karts/moveable.hpp"
 #include "karts/controller/kart_control.hpp"
-#include "karts/player_difficulty.hpp"
 #include "race/race_manager.hpp"
 
 namespace irr
@@ -39,12 +41,14 @@ class btKart;
 class btQuaternion;
 class Controller;
 class Item;
+class KartGFX;
 class KartModel;
 class KartProperties;
 class Material;
 class Powerup;
 class Skidding;
 class SlipStream;
+class TerrainInfo;
 
 /** An abstract interface for the actual karts. Some functions are actually
  *  implemented here in order to allow inlining.
@@ -70,10 +74,10 @@ private:
 
 protected:
     /** The kart properties. */
-    const KartProperties *m_kart_properties;
+    std::unique_ptr<KartProperties> m_kart_properties;
 
     /** The per-player difficulty. */
-    const PlayerDifficulty *m_difficulty;
+    PerPlayerDifficulty m_difficulty;
 
     /** This stores a copy of the kart model. It has to be a copy
      *  since otherwise incosistencies can happen if the same kart
@@ -95,7 +99,8 @@ public:
                    AbstractKart(const std::string& ident,
                                 int world_kart_id,
                                 int position, const btTransform& init_transform,
-                                const PlayerDifficulty *difficulty);
+                                PerPlayerDifficulty difficulty,
+                                video::E_RENDER_TYPE rt);
     virtual       ~AbstractKart();
     virtual core::stringw getName() const;
     virtual void   reset();
@@ -120,20 +125,17 @@ public:
     // ------------------------------------------------------------------------
     /** Returns the kart properties of this kart. */
     const KartProperties* getKartProperties() const
-                            { return m_kart_properties; }
-    // ------------------------------------------------------------------------
-    /** Sets the kart properties. */
-    void setKartProperties(const KartProperties *kp) { m_kart_properties=kp; }
+                            { return m_kart_properties.get(); }
 
     // ========================================================================
     // Access to the per-player difficulty.
     // ------------------------------------------------------------------------
     /** Returns the per-player difficulty of this kart. */
-    const PlayerDifficulty* getPlayerDifficulty() const
+    const PerPlayerDifficulty getPerPlayerDifficulty() const
                             { return m_difficulty; }
     // ------------------------------------------------------------------------
     /** Sets the per-player difficulty. */
-    void setPlayerDifficulty(const PlayerDifficulty *pd) { m_difficulty=pd; }
+    void setPerPlayerDifficulty(const PerPlayerDifficulty d) { m_difficulty=d; }
 
     // ------------------------------------------------------------------------
     /** Returns a unique identifier for this kart (name of the directory the
@@ -144,6 +146,10 @@ public:
      *  speed. */
     virtual float getMaxSteerAngle () const = 0;
     // ------------------------------------------------------------------------
+    /** Returns the (maximum) speed for a given turn radius.
+     *  \param radius The radius for which the speed needs to be computed. */
+    virtual float  getSpeedForTurnRadius(float radius) const = 0;
+    // ------------------------------------------------------------------------
     /** Returns the time till full steering is reached for this kart.
      *  This can depend on the current steering value, which must be >= 0.
      */
@@ -245,7 +251,7 @@ public:
     /** Marks this kart to be eliminated. */
     virtual void eliminate() = 0;
     // ------------------------------------------------------------------------
-    virtual void finishedRace(float time) = 0;
+    virtual void finishedRace(float time, bool from_server=false) = 0;
     // ------------------------------------------------------------------------
     /** Returns the finished time for a kart. */
     virtual float getFinishTime() const = 0;
@@ -373,6 +379,9 @@ public:
     /** Returns the current powerup. */
     virtual Powerup *getPowerup() = 0;
     // ------------------------------------------------------------------------
+    /** Returns a points to this kart's graphical effects. */
+    virtual KartGFX* getKartGFX() = 0;
+    // ------------------------------------------------------------------------
     virtual void setPowerup (PowerupManager::PowerupType t, int n) = 0;
     // ------------------------------------------------------------------------
     /** Returns the bullet vehicle which represents this kart. */
@@ -419,6 +428,9 @@ public:
     /** Shows the star effect for a certain time. */
     virtual void showStarEffect(float t) = 0;
     // ------------------------------------------------------------------------
+    /** Returns the terrain info oject. */
+    virtual const TerrainInfo *getTerrainInfo() const = 0;
+    // ------------------------------------------------------------------------
     /** Called when the kart crashes against another kart.
      *  \param k The kart that was hit.
      *  \param update_attachments If true the attachment of this kart and the
@@ -445,6 +457,15 @@ public:
     /** Counter which is used for displaying wrong way message after a delay */
     virtual float getWrongwayCounter() = 0;
     virtual void setWrongwayCounter(float counter) = 0;
+    // ------------------------------------------------------------------------
+    /** Returns whether this kart wins or loses. */
+    virtual bool getRaceResult() const = 0;
+    // ------------------------------------------------------------------------
+    /** Returns whether this kart is a ghost (replay) kart. */
+    virtual bool isGhostKart() const = 0;
+    // ------------------------------------------------------------------------
+    /** Returns whether this kart is jumping. */
+    virtual bool isJumping() const = 0;
 
 };   // AbstractKart
 
