@@ -19,6 +19,7 @@
 #include "karts/kart_model.hpp"
 
 #include <IMeshSceneNode.h>
+#include <SMesh.h>
 #include <ISceneManager.h>
 
 #include "config/stk_config.hpp"
@@ -138,6 +139,7 @@ KartModel::KartModel(bool is_master)
         m_animation_frame[i]=-1;
     m_animation_speed   = 25;
     m_current_animation = AF_DEFAULT;
+    m_play_non_loop     = false;
 }   // KartModel
 
 // ----------------------------------------------------------------------------
@@ -225,6 +227,16 @@ KartModel::~KartModel()
             // Master KartModels should never have a wheel attached.
             assert(!m_is_master);
 
+            // Drop the cloned transparent mesh wheel if created
+            if (m_wheel_model[i])
+            {
+                // m_wheel_model[i] can be NULL
+                if (m_wheel_model[i]
+                    ->getRenderType() == video::ERT_TRANSPARENT)
+                {
+                    m_wheel_model[i]->drop();
+                }
+            }
             m_wheel_node[i]->drop();
         }
         if(m_is_master && m_wheel_model[i])
@@ -241,6 +253,12 @@ KartModel::~KartModel()
             // Master KartModels should never have a speed weighted object attached.
             assert(!m_is_master);
 
+            // Drop the cloned model if created
+            if (m_speed_weighted_objects[i].m_model
+                ->getRenderType() != video::ERT_DEFAULT)
+            {
+                m_speed_weighted_objects[i].m_model->drop();
+            }
             m_speed_weighted_objects[i].m_node->drop();
         }
         if(m_is_master && m_speed_weighted_objects[i].m_model)
@@ -280,7 +298,7 @@ KartModel::~KartModel()
  *  It is also marked not to be a master copy, so attachModel can be called
  *  for this instance.
  */
-KartModel* KartModel::makeCopy()
+KartModel* KartModel::makeCopy(video::E_RENDER_TYPE rt)
 {
     // Make sure that we are copying from a master objects, and
     // that there is indeed no animated node defined here ...
@@ -293,7 +311,7 @@ KartModel* KartModel::makeCopy()
     km->m_kart_height       = m_kart_height;
     km->m_kart_highest_point= m_kart_highest_point;
     km->m_kart_lowest_point = m_kart_lowest_point;
-    km->m_mesh              = irr_driver->copyAnimatedMesh(m_mesh);
+    km->m_mesh              = irr_driver->copyAnimatedMesh(m_mesh, rt);
     km->m_model_filename    = m_model_filename;
     km->m_animation_speed   = m_animation_speed;
     km->m_current_animation = AF_DEFAULT;
@@ -304,12 +322,23 @@ KartModel* KartModel::makeCopy()
     km->m_nitro_emitter_position[0] = m_nitro_emitter_position[0];
     km->m_nitro_emitter_position[1] = m_nitro_emitter_position[1];
     km->m_has_nitro_emitter = m_has_nitro_emitter;
-    
+
+    scene::IMeshManipulator *mani =
+        irr_driver->getVideoDriver()->getMeshManipulator();
     for(unsigned int i=0; i<4; i++)
     {
-        km->m_wheel_model[i]             = m_wheel_model[i];
         // Master should not have any wheel nodes.
         assert(!m_wheel_node[i]);
+        if (m_wheel_model[i] && rt == video::ERT_TRANSPARENT)
+        {
+            // Only clone the mesh if transparence is used
+            scene::SMesh* clone = mani->createMeshCopy(m_wheel_model[i]);
+            clone->setRenderType(rt);
+            km->m_wheel_model[i] = dynamic_cast<scene::IMesh*>(clone);
+        }
+        else
+            km->m_wheel_model[i] = m_wheel_model[i];
+
         km->m_wheel_filename[i]             = m_wheel_filename[i];
         km->m_wheel_graphics_position[i]    = m_wheel_graphics_position[i];
         km->m_wheel_graphics_radius[i]      = m_wheel_graphics_radius[i];
@@ -324,6 +353,12 @@ KartModel* KartModel::makeCopy()
         // Master should not have any speed weighted nodes.
         assert(!m_speed_weighted_objects[i].m_node);
         km->m_speed_weighted_objects[i] = m_speed_weighted_objects[i];
+        if (rt != video::ERT_DEFAULT)
+        {
+            // Only clone the mesh if default type is not used
+            km->m_speed_weighted_objects[i].m_model = irr_driver
+                ->copyAnimatedMesh(m_speed_weighted_objects[i].m_model, rt);
+        }
     }
 
     for(unsigned int i=AF_BEGIN; i<=AF_END; i++)
@@ -654,7 +689,7 @@ void KartModel::reset()
             m_wheel_node[i]->setRotation(rotation);
         }
     }
-    update(0.0f, 0.0f, 0.0f, 0.0f);
+    update(0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
 
     // Stop any animations currently being played.
     setAnimation(KartModel::AF_DEFAULT);
@@ -682,11 +717,12 @@ void KartModel::finishedRace()
 /** Enables- or disables the end animation.
  *  \param type The type of animation to play.
  */
-void KartModel::setAnimation(AnimationFrameType type)
+void KartModel::setAnimation(AnimationFrameType type, bool play_non_loop)
 {
     // if animations disabled, give up
     if (m_animated_node == NULL) return;
 
+    m_play_non_loop = play_non_loop;
     m_current_animation = type;
     if(m_current_animation==AF_DEFAULT)
     {
@@ -715,6 +751,11 @@ void KartModel::setAnimation(AnimationFrameType type)
         m_animated_node->setLoopMode(false);
         m_animated_node->setAnimationEndCallback(this);
     }
+    else
+    {
+        // Special animation not found, revert to default
+        m_current_animation = AF_DEFAULT;
+    }
 }   // setAnimation
 
 // ----------------------------------------------------------------------------
@@ -767,6 +808,14 @@ void KartModel::OnAnimationEnd(scene::IAnimatedMeshSceneNode *node)
 // ----------------------------------------------------------------------------
 void KartModel::setDefaultSuspension()
 {
+    GhostKart* gk = dynamic_cast<GhostKart*>(m_kart);
+    if (gk)
+    {
+        for (int i = 0; i < 4; i++)
+            m_default_physics_suspension[i] = gk->getSuspensionLength(0, i);
+        return;
+    }
+
     for(int i=0; i<m_kart->getVehicle()->getNumWheels(); i++)
     {
         const btWheelInfo &wi = m_kart->getVehicle()->getWheelInfo(i);
@@ -784,31 +833,62 @@ void KartModel::setDefaultSuspension()
  *  \param suspension Suspension height for all four wheels.
  *  \param speed The speed of the kart in meters/sec, used for the
  *         speed-weighted objects' animations
+ *  \param current_lean_angle How much the kart is leaning (positive meaning
+ *         left side down)
+ *  \param gt_replay_index The index to get replay data, used by ghost kart
  */
-void KartModel::update(float dt, float distance, float steer,  float speed)
+void KartModel::update(float dt, float distance, float steer, float speed,
+                       float current_lean_angle, int gt_replay_index)
 {
     core::vector3df wheel_steer(0, steer*30.0f, 0);
 
     for(unsigned int i=0; i<4; i++)
     {
-       if (!m_kart || !m_wheel_node[i]) continue;
+        if (!m_kart || !m_wheel_node[i]) continue;
 #ifdef DEBUG
-       if (UserConfigParams::m_physics_debug && 
-           !dynamic_cast<GhostKart*>(m_kart)     )
-       {
-           const btWheelInfo &wi = m_kart->getVehicle()->getWheelInfo(i);
-           // Make wheels that are not touching the ground invisible
-           m_wheel_node[i]->setVisible(wi.m_raycastInfo.m_isInContact);
-       }
+        if (UserConfigParams::m_physics_debug &&
+            !m_kart->isGhostKart())
+        {
+            const btWheelInfo &wi = m_kart->getVehicle()->getWheelInfo(i);
+            // Make wheels that are not touching the ground invisible
+            m_wheel_node[i]->setVisible(wi.m_raycastInfo.m_isInContact);
+        }
 #endif
         core::vector3df pos =  m_wheel_graphics_position[i].toIrrVector();
 
-        const btWheelInfo &wi = m_kart->getVehicle()->getWheelInfo(i);
+        float suspension_length = 0.0f;
+        GhostKart* gk = dynamic_cast<GhostKart*>(m_kart);
+        // Prevent using m_default_physics_suspension uninitialized
+        if (gk && gt_replay_index == -1) break;
+
+        if (gk)
+        {
+            suspension_length = gk->getSuspensionLength(gt_replay_index, i);
+        }
+        else
+        {
+            suspension_length = m_kart->getVehicle()->getWheelInfo(i).
+                                m_raycastInfo.m_suspensionLength;
+        }
 
         // Check documentation of Kart::updateGraphics for the following line
         pos.Y +=   m_default_physics_suspension[i]
-                 - wi.m_raycastInfo.m_suspensionLength
+                 - suspension_length
                  - m_kart_lowest_point;
+
+        // Adjust the wheel position for lean: the lean can cause the 'leaned
+        // to' side to be partly in the ground - which looks ok (like the tyres
+        // being compressed), but the other side will be in the air. To avoid
+        // this, increase the position of the wheels on the side that are
+        // higher in the ground so that the wheel still touch the ground.
+        if(current_lean_angle > 0 && (i&1) == 0)   // i&1 == 0: left side
+        {
+            pos.Y -= 2*current_lean_angle;
+        }
+        else if (current_lean_angle < 0 && (i&1) == 1)   // i&1 == 1: right side
+        {
+            pos.Y += 2*current_lean_angle;
+        }
         m_wheel_node[i]->setPosition(pos);
 
         // Now calculate the new rotation: (old + change) mod 360
@@ -825,6 +905,12 @@ void KartModel::update(float dt, float distance, float steer,  float speed)
     // If animations are disabled, stop here
     if (m_animated_node == NULL) return;
 
+    if (m_play_non_loop && m_animated_node->getLoopMode() == true)
+    {
+        m_play_non_loop = false;
+        this->setAnimation(AF_DEFAULT);
+    }
+
     // Update the speed-weighted objects' animations
     if (m_kart != NULL)
     {
