@@ -122,8 +122,6 @@
 #    include <unistd.h>
 #  endif
 #  define WIN32_LEAN_AND_MEAN
-#  define _WINSOCKAPI_
-#  define WIN32_LEAN_AND_MEAN
 #  include <windows.h>
 #  ifdef _MSC_VER
 #    include <direct.h>
@@ -152,6 +150,7 @@
 #include "config/player_profile.hpp"
 #include "config/stk_config.hpp"
 #include "config/user_config.hpp"
+#include "graphics/camera.hpp"
 #include "graphics/central_settings.hpp"
 #include "graphics/graphics_restrictions.hpp"
 #include "graphics/irr_driver.hpp"
@@ -169,22 +168,18 @@
 #include "items/attachment_manager.hpp"
 #include "items/item_manager.hpp"
 #include "items/projectile_manager.hpp"
-#include "karts/controller/ai_base_controller.hpp"
+#include "karts/controller/ai_base_lap_controller.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/kart_properties_manager.hpp"
+#include "modes/cutscene_world.hpp"
 #include "modes/demo_world.hpp"
 #include "modes/profile_world.hpp"
-#include "network/client_network_manager.hpp"
-#include "network/network_manager.hpp"
-#include "network/protocol_manager.hpp"
-#include "network/protocols/server_lobby_room_protocol.hpp"
-#include "network/client_network_manager.hpp"
-#include "network/server_network_manager.hpp"
-#include "network/protocol_manager.hpp"
-#include "network/protocols/server_lobby_room_protocol.hpp"
+#include "network/network_config.hpp"
+#include "network/network_string.hpp"
+#include "network/servers_manager.hpp"
+#include "network/stk_host.hpp"
 #include "online/profile_manager.hpp"
 #include "online/request_manager.hpp"
-#include "online/servers_manager.hpp"
 #include "race/grand_prix_manager.hpp"
 #include "race/highscore_manager.hpp"
 #include "race/history.hpp"
@@ -474,12 +469,14 @@ void setupRaceStart()
         Log::warn("main", "Kart '%s' is unknown so will use the "
             "default kart.",
             UserConfigParams::m_default_kart.c_str());
-        race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart.getDefaultValue());
+        race_manager->setPlayerKart(0, 
+                           UserConfigParams::m_default_kart.getDefaultValue());
     }
     else
     {
         // Set up race manager appropriately
-        race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart);
+        if (race_manager->getNumPlayers() > 0)
+            race_manager->setPlayerKart(0, UserConfigParams::m_default_kart);
     }
 
     // ASSIGN should make sure that only input from assigned devices
@@ -526,19 +523,22 @@ void cmdLineHelp()
     "       --profile-time=n   Enable automatic driven profile mode for n "
                               "seconds.\n"
     "       --no-graphics      Do not display the actual race.\n"
-    "       --with-profile     Enables the profile mode.\n"
     "       --demo-mode=t      Enables demo mode after t seconds idle time in "
                                "main menu.\n"
     "       --demo-tracks=t1,t2 List of tracks to be used in demo mode. No\n"
     "                          spaces are allowed in the track names.\n"
     "       --demo-laps=n      Number of laps in a demo.\n"
     "       --demo-karts=n     Number of karts to use in a demo.\n"
-    "       --ghost            Replay ghost data together with one player kart.\n"
     // "       --history          Replay history file 'history.dat'.\n"
     // "       --history=n        Replay history file 'history.dat' using:\n"
     // "                            n=1: recorded positions\n"
     // "                            n=2: recorded key strokes\n"
-    "       --server           Start a server (not a playing client).\n"
+    // "       --test-ai=n        Use the test-ai for every n-th AI kart.\n"
+    // "                          (so n=1 means all Ais will be the test ai)\n"
+    // "                             
+    "       --server=name      Start a server (not a playing client).\n"
+    "       --lan-server=name  Start a LAN server (not a playing client).\n"
+    "       --server-password= Sets a password for a server (both client&server).\n"
     "       --login=s          Automatically log in (set the login).\n"
     "       --password=s       Automatically log in (set the password).\n"
     "       --port=n           Port number to use.\n"
@@ -552,6 +552,8 @@ void cmdLineHelp()
     "       --root=DIR         Path to add to the list of STK root directories.\n"
     "                          You can specify more than one by separating them\n"
     "                          with colons (:).\n"
+    "       --cutscene=NAME    Launch the specified track as a cutscene.\n"
+    "                          This is for internal debugging use only.\n"
     "\n"
     "You can visit SuperTuxKart's homepage at "
     "http://supertuxkart.sourceforge.net\n\n",
@@ -722,6 +724,12 @@ int handleCmdLinePreliminary()
     if (CommandLine::has("--easter", &n))
         UserConfigParams::m_easter_ear_mode = n;
 
+    // Useful for debugging: the temple navmesh needs 12 minutes in debug
+    // mode to compute the distance matrix!!
+    if(CommandLine::has("--dont-load-navmesh"))
+        Track::m_dont_load_navmesh = true;
+
+
     return 0;
 }   // handleCmdLinePreliminary
 
@@ -757,20 +765,24 @@ int handleCmdLine()
     if(CommandLine::has("--ftl-debug"))
         UserConfigParams::m_ftl_debug = true;
     if(CommandLine::has("--slipstream-debug"))
-            UserConfigParams::m_slipstream_debug=true;
+        UserConfigParams::m_slipstream_debug = true;
     if(CommandLine::has("--rendering-debug"))
         UserConfigParams::m_rendering_debug=true;
     if(CommandLine::has("--ai-debug"))
         AIBaseController::enableDebug();
+    if(CommandLine::has("--test-ai", &n))
+        AIBaseController::setTestAI(n);
+    if (CommandLine::has("--fps-debug"))
+        UserConfigParams::m_fps_debug = true;
 
     if(UserConfigParams::m_artist_debug_mode)
     {
        if(CommandLine::has("--camera-wheel-debug"))
-           UserConfigParams::m_camera_debug=2;
+           Camera::setDebugMode(Camera::CM_DEBUG_GROUND);
         if(CommandLine::has("--camera-debug"))
-            UserConfigParams::m_camera_debug=1;
+            Camera::setDebugMode(Camera::CM_DEBUG_TOP_OF_KART);
         if(CommandLine::has("--camera-kart-debug"))
-            UserConfigParams::m_camera_debug=4;
+            Camera::setDebugMode(Camera::CM_DEBUG_BEHIND_KART);
         if(CommandLine::has("--physics-debug"))
             UserConfigParams::m_physics_debug=1;
         if(CommandLine::has("--check-debug"))
@@ -778,15 +790,35 @@ int handleCmdLine()
     }
 
     // Networking command lines
-    if(CommandLine::has("--server") )
+    NetworkConfig::get()->
+        setMaxPlayers(UserConfigParams::m_server_max_players);
+    if(CommandLine::has("--server", &s))
     {
-        NetworkManager::getInstance<ServerNetworkManager>();
-        Log::info("main", "Creating a server network manager.");
-    }   // -server
+        NetworkConfig::get()->setServerName(core::stringw(s.c_str()));
+        NetworkConfig::get()->setIsServer(true);
+        NetworkConfig::get()->setIsWAN();
+        STKHost::create();
+        Log::info("main", "Creating a WAN server '%s'.", s.c_str());
+    } 
+    if (CommandLine::has("--lan-server", &s))
+    {
+        NetworkConfig::get()->setServerName(core::stringw(s.c_str()));
+        NetworkConfig::get()->setIsServer(true);
+        NetworkConfig::get()->setIsLAN();
+        STKHost::create();
+        Log::info("main", "Creating a LAN server '%s'.", s.c_str());
+    }   
+    if (CommandLine::has("--server-password", &s))
+    {
+        NetworkConfig::get()->setPassword(s);
+    }
 
     if(CommandLine::has("--max-players", &n))
         UserConfigParams::m_server_max_players=n;
 
+    if(CommandLine::has("--start-console"))
+        STKHost::m_enable_console = true;
+
     if(CommandLine::has("--login", &s) )
     {
         login = s.c_str();
@@ -799,8 +831,8 @@ int handleCmdLine()
     // Race parameters
     if(CommandLine::has("--kartsize-debug"))
     {
-        for(unsigned int i=0;
-            i<kart_properties_manager->getNumberOfKarts(); i++)
+        for(unsigned int i=0; i<kart_properties_manager->getNumberOfKarts();
+           i++)
         {
             const KartProperties *km =
                 kart_properties_manager->getKartById(i);
@@ -828,7 +860,7 @@ int handleCmdLine()
             // up upon player creation.
             if (StateManager::get()->activePlayerCount() > 0)
             {
-                race_manager->setLocalKartInfo(0, s);
+                race_manager->setPlayerKart(0, s);
             }
             Log::verbose("main", "You chose to use kart '%s'.",
                          s.c_str());
@@ -904,6 +936,17 @@ int handleCmdLine()
         }
     }   // --track
 
+    // used only for debugging/testing
+    if (CommandLine::has("--cutscene", &s))
+    {
+        race_manager->setTrack(s);
+        StateManager::get()->enterGameState();
+        race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
+        race_manager->setNumKarts(0);
+        race_manager->setNumPlayers(0);
+        race_manager->setNumLaps(999);
+    } // --cutscene
+
     if(CommandLine::has("--gp", &s))
     {
         race_manager->setMajorMode(RaceManager::MAJOR_MODE_GRAND_PRIX);
@@ -979,19 +1022,6 @@ int handleCmdLine()
         race_manager->setNumLaps(999999); // profile end depends on time
     }   // --profile-time
 
-    if(CommandLine::has("--with-profile") )
-    {
-        // Set default profile mode of 1 lap if we haven't already set one
-        if (!ProfileWorld::isProfileMode()) {
-            UserConfigParams::m_no_start_screen = true;
-            ProfileWorld::setProfileModeLaps(1);
-            race_manager->setNumLaps(1);
-        }
-    }   // --with-profile
-
-    if(CommandLine::has("--ghost"))
-        ReplayPlay::create();
-
     if(CommandLine::has("--history",  &n))
     {
         history->doReplayHistory( (History::HistoryReplayMode)n);
@@ -1011,7 +1041,7 @@ int handleCmdLine()
     // Demo mode
     if(CommandLine::has("--demo-mode", &s))
     {
-        float t;
+        float t = 0;
         StringUtils::fromString(s, t);
         DemoWorld::enableDemoMode(t);
         // The default number of laps is taken from ProfileWorld and
@@ -1142,6 +1172,7 @@ void initRest()
     // The order here can be important, e.g. KartPropertiesManager needs
     // defaultKartProperties, which are defined in stk_config.
     history                 = new History              ();
+    ReplayPlay::create();
     ReplayRecorder::create();
     material_manager        = new MaterialManager      ();
     track_manager           = new TrackManager         ();
@@ -1159,6 +1190,11 @@ void initRest()
     track_manager->addTrackSearchDir(
                  file_manager->getAddonsFile("tracks/"));
 
+    {
+        XMLNode characteristicsNode(file_manager->getAsset("kart_characteristics.xml"));
+        kart_properties_manager->loadCharacteristics(&characteristicsNode);
+    }
+
     track_manager->loadTrackList();
     music_manager->addMusicToTracks();
 
@@ -1174,7 +1210,7 @@ void initRest()
 
     race_manager            = new RaceManager          ();
     // default settings for Quickstart
-    race_manager->setNumLocalPlayers(1);
+    race_manager->setNumPlayers(1);
     race_manager->setNumLaps   (3);
     race_manager->setMajorMode (RaceManager::MAJOR_MODE_SINGLE);
     race_manager->setMinorMode (RaceManager::MINOR_MODE_NORMAL_RACE);
@@ -1263,9 +1299,9 @@ int main(int argc, char *argv[] )
         handleCmdLineOutputModifier();
 
         if(CommandLine::has("--root", &s))
-        {
             FileManager::addRootDirs(s);
-        }
+        if (CommandLine::has("--stdout", &s))
+            FileManager::setStdoutName(s);
 
         // Init the minimum managers so that user config exists, then
         // handle all command line options that do not need (or must
@@ -1287,14 +1323,6 @@ int main(int argc, char *argv[] )
         main_loop = new MainLoop();
         material_manager->loadMaterial();
 
-        // Load the font textures - they are all lazily loaded
-        // so no need to push a texture search path. They will actually
-        // be loaded from ScalableFont.
-        file_manager->pushTextureSearchPath(file_manager->getAsset(FileManager::FONT, ""));
-        material_manager->addSharedMaterial(
-                   file_manager->getAsset(FileManager::FONT,"materials.xml"));
-        file_manager->popTextureSearchPath();
-
         GUIEngine::addLoadingIcon( irr_driver->getTexture(FileManager::GUI,
                                                           "options_video.png"));
         kart_properties_manager -> loadAllKarts    ();
@@ -1348,20 +1376,6 @@ int main(int argc, char *argv[] )
         //handleCmdLine() needs InitTuxkart() so it can't be called first
         if(!handleCmdLine()) exit(0);
 
-        // load the network manager
-        // If the server has been created (--server option), this will do nothing (just a warning):
-        NetworkManager::getInstance<ClientNetworkManager>();
-        if (NetworkManager::getInstance()->isServer())
-        {
-            ServerNetworkManager::getInstance()->setMaxPlayers(
-                    UserConfigParams::m_server_max_players);
-        }
-        NetworkManager::getInstance()->run();
-        if (NetworkManager::getInstance()->isServer())
-        {
-            ProtocolManager::getInstance()->requestStart(new ServerLobbyRoomProtocol());
-        }
-
         addons_manager->checkInstalledAddons();
 
         // Load addons.xml to get info about addons even when not
@@ -1529,7 +1543,9 @@ int main(int argc, char *argv[] )
     // so we don't crash later when StateManager tries to access input devices.
     StateManager::get()->resetActivePlayers();
     if(input_manager) delete input_manager; // if early crash avoid delete NULL
-    NetworkManager::getInstance()->abort();
+
+    if(NetworkConfig::get()->isNetworking() && STKHost::existHost())
+        STKHost::get()->abort();
 
     cleanSuperTuxKart();
 
@@ -1581,7 +1597,6 @@ static void cleanSuperTuxKart()
     irr_driver->updateConfigIfRelevant();
     AchievementsManager::destroy();
     Referee::cleanup();
-    if(ReplayPlay::get())       ReplayPlay::destroy();
     if(race_manager)            delete race_manager;
     if(grand_prix_manager)      delete grand_prix_manager;
     if(highscore_manager)       delete highscore_manager;
@@ -1593,6 +1608,7 @@ static void cleanSuperTuxKart()
     if(track_manager)           delete track_manager;
     if(material_manager)        delete material_manager;
     if(history)                 delete history;
+    ReplayPlay::destroy();
     ReplayRecorder::destroy();
     delete ParticleKindManager::get();
     PlayerManager::destroy();
@@ -1637,8 +1653,9 @@ static void cleanSuperTuxKart()
 
     // FIXME: do we need to wait for threads there, can they be
     // moved further up?
-    Online::ServersManager::deallocate();
-    NetworkManager::kill();
+    ServersManager::deallocate();
+    if(NetworkConfig::get()->isNetworking() && STKHost::existHost())
+        STKHost::destroy();
 
     cleanUserConfig();
 
@@ -1668,7 +1685,14 @@ static void cleanUserConfig()
 //=============================================================================
 void runUnitTests()
 {
+    Log::info("UnitTest", "Starting unit testing");
+    Log::info("UnitTest", "=====================");
+    Log::info("UnitTest", "- GraphicsRestrictions");
     GraphicsRestrictions::unitTesting();
+    Log::info("UnitTest", " - NetworkString");
+    NetworkString::unitTesting();
+
+    Log::info("UnitTest", " - Easter detection");
     // Test easter mode: in 2015 Easter is 5th of April - check with 0 days
     // before and after
     int saved_easter_mode = UserConfigParams::m_easter_ear_mode;
@@ -1698,4 +1722,7 @@ void runUnitTests()
     assert( isEasterMode(22, 3, 2016, 5));
     assert(!isEasterMode(21, 3, 2016, 5));
     UserConfigParams::m_easter_ear_mode = saved_easter_mode;
-}   // unitTesting
+    Log::info("UnitTest", "=====================");
+    Log::info("UnitTest", "Testing successful   ");
+    Log::info("UnitTest", "=====================");
+}   // runUnitTests
