@@ -30,8 +30,10 @@
 #include "input/wiimote_manager.hpp"
 #include "modes/profile_world.hpp"
 #include "modes/world.hpp"
+#include "network/network_config.hpp"
 #include "network/protocol_manager.hpp"
-#include "network/network_world.hpp"
+#include "network/race_event_manager.hpp"
+#include "network/stk_host.hpp"
 #include "online/request_manager.hpp"
 #include "race/race_manager.hpp"
 #include "states_screens/state_manager.hpp"
@@ -59,6 +61,12 @@ MainLoop::~MainLoop()
  */
 float MainLoop::getLimitedDt()
 {
+    // In profile mode without graphics, run with a fixed dt of 1/60
+    if (ProfileWorld::isProfileMode() && ProfileWorld::isNoGraphics())
+    {
+        return 1.0f/60.0f;
+    }
+
     IrrlichtDevice* device = irr_driver->getDevice();
     m_prev_time = m_curr_time;
 
@@ -67,6 +75,24 @@ float MainLoop::getLimitedDt()
     {
         m_curr_time = device->getTimer()->getRealTime();
         dt = (float)(m_curr_time - m_prev_time);
+        const World* const world = World::getWorld();
+        if (UserConfigParams::m_fps_debug && world)
+        {
+            const LinearWorld *lw = dynamic_cast<const LinearWorld*>(world);
+            if (lw)
+            {
+                Log::verbose("fps", "time %f distance %f dt %f fps %f",
+                             lw->getTime(),
+                             lw->getDistanceDownTrackForKart(0),
+                             dt*0.001f, 1000.0f / dt);
+            }
+            else
+            {
+                Log::verbose("fps", "time %f dt %f fps %f",
+                             world->getTime(), dt*0.001f, 1000.0f / dt);
+            }
+
+        }
 
         // don't allow the game to run slower than a certain amount.
         // when the computer can't keep it up, slow down the shown time instead
@@ -99,10 +125,9 @@ float MainLoop::getLimitedDt()
  */
 void MainLoop::updateRace(float dt)
 {
-    if(ProfileWorld::isProfileMode()) dt=1.0f/60.0f;
-
-    if (NetworkWorld::getInstance<NetworkWorld>()->isRunning())
-        NetworkWorld::getInstance<NetworkWorld>()->update(dt);
+    // The race event manager will update world in case of an online race
+    if (RaceEventManager::getInstance<RaceEventManager>()->isRunning())
+        RaceEventManager::getInstance<RaceEventManager>()->update(dt);
     else
         World::getWorld()->updateWorld(dt);
 }   // updateRace
@@ -158,7 +183,13 @@ void MainLoop::run()
             PROFILER_POP_CPU_MARKER();
 
             PROFILER_PUSH_CPU_MARKER("Protocol manager update", 0x7F, 0x00, 0x7F);
-            ProtocolManager::getInstance()->update();
+            if (STKHost::existHost())
+            {
+                if (STKHost::get()->requestedShutdown())
+                    STKHost::get()->shutdown();
+                else
+                    ProtocolManager::getInstance()->update(dt);
+            }
             PROFILER_POP_CPU_MARKER();
 
             PROFILER_PUSH_CPU_MARKER("Database polling update", 0x00, 0x7F, 0x7F);
@@ -168,7 +199,8 @@ void MainLoop::run()
         else if (!m_abort && ProfileWorld::isNoGraphics())
         {
             PROFILER_PUSH_CPU_MARKER("Protocol manager update", 0x7F, 0x00, 0x7F);
-            ProtocolManager::getInstance()->update();
+            if(NetworkConfig::get()->isNetworking())
+                ProtocolManager::getInstance()->update(dt);
             PROFILER_POP_CPU_MARKER();
 
             PROFILER_PUSH_CPU_MARKER("Database polling update", 0x00, 0x7F, 0x7F);
