@@ -58,7 +58,8 @@ CutsceneWorld::CutsceneWorld() : World()
     m_aborted = false;
     WorldStatus::setClockMode(CLOCK_NONE);
     m_use_highscores = false;
-    m_play_racestart_sounds = false;
+    m_play_track_intro_sound = false;
+    m_play_ready_set_go_sounds = false;
     m_fade_duration = 1.0f;
 }   // CutsceneWorld
 
@@ -421,7 +422,6 @@ void CutsceneWorld::enterRaceOverState()
                 race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
                 race_manager->setNumKarts(0);
                 race_manager->setNumPlayers(0);
-                race_manager->setNumLocalPlayers(0);
                 race_manager->startSingleRace("featunlocked", 999, race_manager->raceWasStartedFromOverworld());
 
                 FeatureUnlockedCutScene* scene =
@@ -474,7 +474,6 @@ void CutsceneWorld::enterRaceOverState()
                 race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
                 race_manager->setNumKarts(0);
                 race_manager->setNumPlayers(0);
-                race_manager->setNumLocalPlayers(0);
                 race_manager->startSingleRace("featunlocked", 999, race_manager->raceWasStartedFromOverworld());
 
                 FeatureUnlockedCutScene* scene =
