@@ -48,7 +48,7 @@ public:
     // overriding World methods
     virtual void reset() OVERRIDE;
     virtual const std::string& getIdent() const OVERRIDE;
-    virtual const btTransform &getStartTransform(int index);
+    virtual const btTransform &getStartTransform(int index) OVERRIDE;
     virtual float getClockStartTime() const;
     virtual void getKartsDisplayInfo(
                  std::vector<RaceGUIBase::KartIconDisplayInfo> *info) OVERRIDE;
