@@ -178,9 +178,10 @@ void LinearWorld::update(float dt)
         // in the position of the kart (e.g. while falling the kart
         // might get too close to another part of the track, shortly
         // jump to position one, then on reset fall back to last)
-        if (!kart_info.getTrackSector()->isOnRoad() &&
+        if ((!kart_info.getTrackSector()->isOnRoad() &&
             (!kart->getMaterial() ||
-              kart->getMaterial()->isDriveReset())     )
+              kart->getMaterial()->isDriveReset()))  &&
+             !kart->isGhostKart())
             continue;
         kart_info.getTrackSector()->update(kart->getFrontXYZ());
         kart_info.m_overall_distance = kart_info.m_race_lap
@@ -247,9 +248,8 @@ void LinearWorld::newLap(unsigned int kart_index)
     AbstractKart *kart  = m_karts[kart_index];
 
     // Reset reset-after-lap achievements
-    StateManager::ActivePlayer *c = kart->getController()->getPlayer();
     PlayerProfile *p = PlayerManager::getCurrentPlayer();
-    if (c && c->getConstProfile() == p)
+    if (kart->getController()->canGetAchievements())
     {
         p->getAchievementsStatus()->onLapEnd();
     }
@@ -282,7 +282,7 @@ void LinearWorld::newLap(unsigned int kart_index)
     if (raceHasLaps() && kart_info.m_race_lap+1 == lap_count)
     {
         m_race_gui->addMessage(_("Final lap!"), kart,
-                               3.0f, video::SColor(255, 210, 100, 50), true);
+                               3.0f, GUIEngine::getSkin()->getColor("font::normal"), true);
         if(!m_last_lap_sfx_played && lap_count > 1)
         {
             if (UserConfigParams::m_music)
@@ -309,7 +309,7 @@ void LinearWorld::newLap(unsigned int kart_index)
              kart_info.m_race_lap+1 < lap_count)
     {
         m_race_gui->addMessage(_("Lap %i", kart_info.m_race_lap+1),
-                               kart, 3.0f, video::SColor(255, 210, 100, 50),
+                               kart, 3.0f, GUIEngine::getSkin()->getColor("font::normal"),
                                true);
     }
 
@@ -840,7 +840,7 @@ void LinearWorld::updateRacePosition()
  */
 void LinearWorld::checkForWrongDirection(unsigned int i, float dt)
 {
-    if (!m_karts[i]->getController()->isPlayerController()) 
+    if (!m_karts[i]->getController()->isLocalPlayerController()) 
         return;
 
     float wrongway_counter = m_karts[i]->getWrongwayCounter();
