@@ -39,7 +39,8 @@ class ThreeStrikesBattle : public WorldWithRank
 private:
     struct BattleInfo
     {
-        int m_lives;
+        int  m_lives;
+        int  m_on_node;
     };
 
     /** This vector contains an 'BattleInfo' struct for every kart in the race.
@@ -70,6 +71,9 @@ private:
 
     PtrVector<TrackObject, REF> m_tires;
 
+    /** Function to update the locations of all karts on the polygon map */
+    void updateKartNodes();
+
 public:
 
     /** Used to show a nice graph when battle is over */
@@ -105,6 +109,7 @@ public:
 
     virtual void kartAdded(AbstractKart* kart, scene::ISceneNode* node);
 
+    int getKartNode(unsigned int kart_id) const;
 
     void updateKartRanks();
 };   // ThreeStrikesBattles
