@@ -31,14 +31,19 @@
 #include "input/device_manager.hpp"
 #include "input/keyboard_device.hpp"
 #include "items/projectile_manager.hpp"
-#include "karts/controller/player_controller.hpp"
+#include "karts/controller/battle_ai.hpp"
+#include "karts/controller/soccer_ai.hpp"
 #include "karts/controller/end_controller.hpp"
+#include "karts/controller/local_player_controller.hpp"
 #include "karts/controller/skidding_ai.hpp"
+#include "karts/controller/test_ai.hpp"
 #include "karts/controller/network_player_controller.hpp"
 #include "karts/kart.hpp"
 #include "karts/kart_properties_manager.hpp"
 #include "modes/overworld.hpp"
 #include "modes/profile_world.hpp"
+#include "modes/soccer_world.hpp"
+#include "network/network_config.hpp"
 #include "physics/btKart.hpp"
 #include "physics/physics.hpp"
 #include "physics/triangle_mesh.hpp"
@@ -144,6 +149,9 @@ void World::init()
     m_eliminated_karts    = 0;
     m_eliminated_players  = 0;
     m_num_players         = 0;
+    unsigned int gk       = 0;
+    if (race_manager->hasGhostKarts())
+        gk = ReplayPlay::get()->getNumGhostKart();
 
     // Create the race gui before anything else is attached to the scene node
     // (which happens when the track is loaded). This allows the race gui to
@@ -177,19 +185,30 @@ void World::init()
     // karts can be positioned properly on (and not in) the tracks.
     m_track->loadTrackModel(race_manager->getReverseTrack());
 
+    if (gk > 0)
+    {
+        ReplayPlay::get()->load();
+        for (unsigned int k = 0; k < gk; k++)
+            m_karts.push_back(ReplayPlay::get()->getGhostKart(k));
+    }
+
+    // Assign team of AIs for soccer mode before createKart
+    SoccerWorld* sw = dynamic_cast<SoccerWorld*>(this);
+    if (sw)
+        sw->setAITeam();
+
     for(unsigned int i=0; i<num_karts; i++)
     {
+        if (race_manager->getKartType(i) == RaceManager::KT_GHOST) continue;
         std::string kart_ident = history->replayHistory()
                                ? history->getKartIdent(i)
                                : race_manager->getKartIdent(i);
         int local_player_id  = race_manager->getKartLocalPlayerId(i);
         int global_player_id = race_manager->getKartGlobalPlayerId(i);
-        const PlayerDifficulty *player_difficulty =
-            stk_config->getPlayerDifficulty(race_manager->getPlayerDifficulty(i));
         AbstractKart* newkart = createKart(kart_ident, i, local_player_id,
                                    global_player_id,
                                    race_manager->getKartType(i),
-                                   player_difficulty);
+                                   race_manager->getPlayerDifficulty(i));
         m_karts.push_back(newkart);
         m_track->adjustForFog(newkart->getNode());
 
@@ -201,11 +220,8 @@ void World::init()
     // Must be called after all karts are created
     m_race_gui->init();
 
-    if(ReplayPlay::get())
-        ReplayPlay::get()->Load();
-
     powerup_manager->updateWeightsForRace(num_karts);
-    
+
     if (UserConfigParams::m_weather_effects)
     {
         m_weather = new Weather(m_track->getWeatherLightning(),
@@ -249,7 +265,7 @@ void World::reset()
         Camera::getCamera(i)->reset();
     }
 
-    if(ReplayPlay::get())
+    if(race_manager->hasGhostKarts())
         ReplayPlay::get()->reset();
 
     resetAllKarts();
@@ -271,7 +287,19 @@ void World::reset()
     race_manager->reset();
     // Make sure to overwrite the data from the previous race.
     if(!history->replayHistory()) history->initRecording();
-    if(ReplayRecorder::get()) ReplayRecorder::get()->init();
+    if(race_manager->isRecordingRace())
+    {
+        Log::info("World", "Start Recording race.");
+        ReplayRecorder::get()->init();
+    }
+    if((NetworkConfig::get()->isServer() && !ProfileWorld::isNoGraphics()) ||
+        race_manager->isWatchingReplay())
+    {
+        // In case that the server is running with gui or watching replay,
+        // create a camera and attach it to the first kart.
+        Camera::createCamera(World::getWorld()->getKart(0));
+
+    }
 
     // Reset all data structures that depend on number of karts.
     irr_driver->reset();
@@ -300,10 +328,14 @@ void World::createRaceGUI()
 AbstractKart *World::createKart(const std::string &kart_ident, int index,
                                 int local_player_id, int global_player_id,
                                 RaceManager::KartType kart_type,
-                                const PlayerDifficulty *difficulty)
+                                PerPlayerDifficulty difficulty)
 {
+    unsigned int gk = 0;
+    if (race_manager->hasGhostKarts())
+        gk = ReplayPlay::get()->getNumGhostKart();
+
     int position           = index+1;
-    btTransform init_pos   = getStartTransform(index);
+    btTransform init_pos   = getStartTransform(index - gk);
     AbstractKart *new_kart = new Kart(kart_ident, index, position, init_pos,
                                       difficulty);
     new_kart->init(race_manager->getKartType(index));
@@ -311,14 +343,12 @@ AbstractKart *World::createKart(const std::string &kart_ident, int index,
     switch(kart_type)
     {
     case RaceManager::KT_PLAYER:
-        controller = new PlayerController(new_kart,
-                         StateManager::get()->getActivePlayer(local_player_id),
-                                          local_player_id);
+        controller = new LocalPlayerController(new_kart,
+                         StateManager::get()->getActivePlayer(local_player_id));
         m_num_players ++;
         break;
     case RaceManager::KT_NETWORK_PLAYER:
-        controller = new NetworkPlayerController(new_kart,
-                        StateManager::get()->getActivePlayer(local_player_id));
+        controller = new NetworkPlayerController(new_kart);
         m_num_players++;
         break;
     case RaceManager::KT_AI:
@@ -351,12 +381,28 @@ Controller* World::loadAIController(AbstractKart *kart)
 {
     Controller *controller;
     int turn=0;
+
+    if(race_manager->getMinorMode()==RaceManager::MINOR_MODE_3_STRIKES)
+        turn=1;
+    else if(race_manager->getMinorMode()==RaceManager::MINOR_MODE_SOCCER)
+        turn=2;
     // If different AIs should be used, adjust turn (or switch randomly
     // or dependent on difficulty)
     switch(turn)
     {
         case 0:
-            controller = new SkiddingAI(kart);
+            // If requested, start the test ai
+            if( (AIBaseController::getTestAI()!=0                       ) && 
+                ( (kart->getWorldKartId()+1) % AIBaseController::getTestAI() )==0)
+                controller = new TestAI(kart);
+            else
+                controller = new SkiddingAI(kart);
+            break;
+        case 1:
+            controller = new BattleAI(kart);
+            break;
+        case 2:
+            controller = new SoccerAI(kart);
             break;
         default:
             Log::warn("[World]", "Unknown AI, using default.");
@@ -372,16 +418,6 @@ World::~World()
 {
     irr_driver->onUnloadWorld();
 
-    if(ReplayPlay::get())
-    {
-        // Destroy the old replay object, which also stored the ghost
-        // karts, and create a new one (which means that in further
-        // races the usage of ghosts will still be enabled).
-        ReplayPlay::destroy();
-        ReplayPlay::create();
-    }
-
-
     // In case that a race is aborted (e.g. track not found) m_track is 0.
     if(m_track)
         m_track->cleanup();
@@ -406,9 +442,29 @@ World::~World()
         delete m_weather;
 
     for ( unsigned int i = 0 ; i < m_karts.size() ; i++ )
+    {
+        // Let ReplayPlay destroy the ghost karts
+        if (m_karts[i]->isGhostKart()) continue;
         delete m_karts[i];
+    }
 
+    if(race_manager->hasGhostKarts() || race_manager->isRecordingRace())
+    {
+        // Destroy the old replay object, which also stored the ghost
+        // karts, and create a new one (which means that in further
+        // races the usage of ghosts will still be enabled).
+        // It can allow auto recreation of ghost replay file lists
+        // when next time visit the ghost replay selection screen.
+        ReplayPlay::destroy();
+        ReplayPlay::create();
+    }
     m_karts.clear();
+    if(race_manager->isRecordingRace())
+        ReplayRecorder::get()->reset();
+    race_manager->setRaceGhostKarts(false);
+    race_manager->setRecordRace(false);
+    race_manager->setWatchingReplay(false);
+
     Camera::removeAllCameras();
 
     projectile_manager->cleanup();
@@ -437,6 +493,7 @@ void World::onGo()
     // from sliding downhill)
     for(unsigned int i=0; i<m_karts.size(); i++)
     {
+        if (m_karts[i]->isGhostKart()) continue;
         m_karts[i]->getVehicle()->setAllBrakes(0);
     }
 }   // onGo
@@ -467,13 +524,10 @@ void World::terminateRace()
     // Update highscores, and retrieve the best highscore if relevant
     // to show it in the GUI
     int best_highscore_rank = -1;
-    int best_finish_time = -1;
     std::string highscore_who = "";
-    StateManager::ActivePlayer* best_player = NULL;
     if (!this->isNetworkWorld())
     {
-        updateHighscores(&best_highscore_rank, &best_finish_time, &highscore_who,
-                     &best_player);
+        updateHighscores(&best_highscore_rank);
     }
 
     // Check achievements
@@ -499,8 +553,7 @@ void World::terminateRace()
         for(unsigned int i = 0; i < kart_amount; i++)
         {
             // Retrieve the current player
-            StateManager::ActivePlayer* p = m_karts[i]->getController()->getPlayer();
-            if (p && p->getConstProfile() == PlayerManager::getCurrentPlayer())
+            if (m_karts[i]->getController()->canGetAchievements())
             {
                 // Check if the player has won
                 if (m_karts[i]->getPosition() == winner_position && kart_amount > opponents )
@@ -524,8 +577,7 @@ void World::terminateRace()
         for(unsigned int i = 0; i < kart_amount; i++)
         {
             // Retrieve the current player
-            StateManager::ActivePlayer* p = m_karts[i]->getController()->getPlayer();
-            if (p && p->getConstProfile() == PlayerManager::getCurrentPlayer())
+            if (m_karts[i]->getController()->canGetAchievements())
             {
                 // Check if the player has won
                 if (m_karts[i]->getPosition() == 1 )
@@ -558,8 +610,7 @@ void World::terminateRace()
 
     if (best_highscore_rank > 0)
     {
-        results->setHighscore(highscore_who, best_player, best_highscore_rank,
-                              best_finish_time);
+        results->setHighscore(best_highscore_rank);
     }
     else
     {
@@ -583,12 +634,13 @@ void World::resetAllKarts()
     getPhysics()->getPhysicsWorld()->resetLocalTime();
 
     // If track checking is requested, check all rescue positions if
-    // they are heigh enough.
+    // they are high enough.
     if(UserConfigParams::m_track_debug)
     {
         // Loop over all karts, in case that some karts are dfferent
         for(unsigned int kart_id=0; kart_id<(unsigned int)m_karts.size(); kart_id++)
         {
+            if (m_karts[kart_id]->isGhostKart()) continue;
             for(unsigned int rescue_pos=0;
                 rescue_pos<getNumberOfRescuePositions();
                 rescue_pos++)
@@ -616,6 +668,7 @@ void World::resetAllKarts()
     //that at least one of its wheel will be on the surface of the track
     for ( KartList::iterator i=m_karts.begin(); i!=m_karts.end(); i++)
     {
+        if ((*i)->isGhostKart()) continue;
         Vec3 xyz = (*i)->getXYZ();
         //start projection from top of kart
         Vec3 up_offset(0, 0.5f * ((*i)->getKartHeight()), 0);
@@ -664,6 +717,7 @@ void World::resetAllKarts()
         all_finished=true;
         for ( KartList::iterator i=m_karts.begin(); i!=m_karts.end(); i++)
         {
+            if ((*i)->isGhostKart()) continue;
             if(!(*i)->isInRest())
             {
                 Vec3            normal;
@@ -857,7 +911,7 @@ void World::updateWorld(float dt)
             if (m_schedule_tutorial)
             {
                 m_schedule_tutorial = false;
-                race_manager->setNumLocalPlayers(1);
+                race_manager->setNumPlayers(1);
                 race_manager->setMajorMode (RaceManager::MAJOR_MODE_SINGLE);
                 race_manager->setMinorMode (RaceManager::MINOR_MODE_TUTORIAL);
                 race_manager->setNumKarts( 1 );
@@ -879,7 +933,7 @@ void World::updateWorld(float dt)
                               UserConfigParams::m_default_kart.c_str());
                     UserConfigParams::m_default_kart.revertToDefaults();
                 }
-                race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart);
+                race_manager->setPlayerKart(0, UserConfigParams::m_default_kart);
 
                 // ASSIGN should make sure that only input from assigned devices
                 // is read.
@@ -942,8 +996,7 @@ void World::update(float dt)
 
     PROFILER_PUSH_CPU_MARKER("World::update (sub-updates)", 0x20, 0x7F, 0x00);
     history->update(dt);
-    if(ReplayRecorder::get()) ReplayRecorder::get()->update(dt);
-    if(ReplayPlay::get()) ReplayPlay::get()->update(dt);
+    if(race_manager->isRecordingRace()) ReplayRecorder::get()->update(dt);
     if(history->replayHistory()) dt=history->getNextDelta();
     WorldStatus::update(dt);
     if (m_script_engine) m_script_engine->update(dt);
@@ -954,7 +1007,7 @@ void World::update(float dt)
         m_physics->update(dt);
     }
 
-    PROFILER_PUSH_CPU_MARKER("World::update (AI)", 0x40, 0x7F, 0x00);
+    PROFILER_PUSH_CPU_MARKER("World::update (Kart::upate)", 0x40, 0x7F, 0x00);
     const int kart_amount = (int)m_karts.size();
     for (int i = 0 ; i < kart_amount; ++i)
     {
@@ -1033,12 +1086,9 @@ Highscores* World::getHighscores() const
  *  score, if so it notifies the HighscoreManager so the new score is added
  *  and saved.
  */
-void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
-                             std::string* highscore_who,
-                             StateManager::ActivePlayer** best_player)
+void World::updateHighscores(int* best_highscore_rank)
 {
     *best_highscore_rank = -1;
-    *best_player = NULL;
 
     if(!m_use_highscores) return;
 
@@ -1079,9 +1129,9 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
             continue;
         }
 
-        // Only record times for player karts and only if
+        // Only record times for local player karts and only if
         // they finished the race
-        if(!m_karts[index[pos]]->getController()->isPlayerController())
+        if(!m_karts[index[pos]]->getController()->isLocalPlayerController())
             continue;
         if (!m_karts[index[pos]]->hasFinishedRace()) continue;
 
@@ -1090,13 +1140,11 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
 
         Highscores* highscores = getHighscores();
 
-        PlayerController *controller = (PlayerController*)(k->getController());
-
         int highscore_rank = 0;
-        if (controller->getPlayer()->getProfile() != NULL) // if we have the player profile here
-            highscore_rank = highscores->addData(k->getIdent(),
-                              controller->getPlayer()->getProfile()->getName(),
-                                                 k->getFinishTime());
+        // The player is a local player, so there is a name:
+        highscore_rank = highscores->addData(k->getIdent(),
+                                             k->getController()->getName(),
+                                             k->getFinishTime()    );
 
         if (highscore_rank > 0)
         {
@@ -1104,9 +1152,6 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
                 highscore_rank < *best_highscore_rank)
             {
                 *best_highscore_rank = highscore_rank;
-                *best_finish_time = (int)(k->getFinishTime());
-                *best_player = controller->getPlayer();
-                *highscore_who = k->getIdent();
             }
 
             highscore_manager->saveHighscores();
@@ -1126,8 +1171,7 @@ AbstractKart *World::getPlayerKart(unsigned int n) const
     unsigned int count=-1;
 
     for(unsigned int i=0; i<m_karts.size(); i++)
-        if(m_karts[i]->getController()->isPlayerController() ||
-            m_karts[i]->getController()->isNetworkController())
+        if(m_karts[i]->getController()->isPlayerController())
         {
             count++;
             if(count==n) return m_karts[i];
@@ -1152,6 +1196,7 @@ AbstractKart *World::getLocalPlayerKart(unsigned int n) const
 void World::eliminateKart(int kart_id, bool notify_of_elimination)
 {
     AbstractKart *kart = m_karts[kart_id];
+    if (kart->isGhostKart()) return;
 
     // Display a message about the eliminated kart in the race guia
     if (notify_of_elimination)
@@ -1170,7 +1215,7 @@ void World::eliminateKart(int kart_id, bool notify_of_elimination)
         }  // for i < number of cameras
     }   // if notify_of_elimination
 
-    if(kart->getController()->isPlayerController())
+    if(kart->getController()->isLocalPlayerController())
     {
         for(unsigned int i=0; i<Camera::getNumCameras(); i++)
         {
@@ -1231,8 +1276,8 @@ void World::unpause()
         // Note that we can not test for isPlayerController here, since
         // an EndController will also return 'isPlayerController' if the
         // kart belonged to a player.
-        PlayerController *pc =
-            dynamic_cast<PlayerController*>(m_karts[i]->getController());
+        LocalPlayerController *pc =
+            dynamic_cast<LocalPlayerController*>(m_karts[i]->getController());
         if(pc)
             pc->resetInputState();
     }
