@@ -115,9 +115,7 @@ protected:
     */
     bool        m_use_highscores;
 
-    void  updateHighscores  (int* best_highscore_rank, int* best_finish_time,
-                             std::string* highscore_who,
-                             StateManager::ActivePlayer** best_player);
+    void  updateHighscores  (int* best_highscore_rank);
     void  resetAllKarts     ();
     void  eliminateKart     (int kart_number, bool notifyOfElimination=true);
     Controller*
@@ -126,7 +124,7 @@ protected:
     virtual AbstractKart *createKart(const std::string &kart_ident, int index,
                              int local_player_id, int global_player_id,
                              RaceManager::KartType type,
-                             const PlayerDifficulty *difficulty);
+                             PerPlayerDifficulty difficulty);
     /** Pointer to the track. The track is managed by world. */
     Track* m_track;
 
@@ -177,7 +175,7 @@ protected:
     Weather* m_weather;
 
 
-    virtual void  onGo();
+    virtual void  onGo() OVERRIDE;
     /** Returns true if the race is over. Must be defined by all modes. */
     virtual bool  isRaceOver() = 0;
     virtual void  update(float dt);
