@@ -17,6 +17,8 @@
 
 #include "network/race_config.hpp"
 
+#include "config/user_config.hpp"
+#include "network/network_config.hpp"
 #include "race/race_manager.hpp"
 #include "utils/log.hpp"
 
@@ -27,8 +29,11 @@
  *  \return The key of type S that has the highest second value.
  */
 template<typename S>
-S getHighestInHistogram(std::map<S,int>* histogram)
+S getHighestInHistogram(std::map<S,int>* histogram, S default_value)
 {
+    if(histogram->empty())
+        return default_value;
+
     S best_item = histogram->begin()->first;
     uint8_t highest_count = histogram->begin()->second;
     for (typename std::map<S, int>::iterator it = histogram->begin();
@@ -41,41 +46,54 @@ S getHighestInHistogram(std::map<S,int>* histogram)
         }
     }
     return best_item;
-}
+}   // getHighestInHistogram
 
 //-----------------------------------------------------------------------------
 //---------------------------------  TrackVote --------------------------------
 //-----------------------------------------------------------------------------
-
+/** Constructor for a track vote.
+ */
 TrackVote::TrackVote()
 {
     has_voted_laps = false;
     has_voted_track = false;
     has_voted_reversed = false;
-}
+}   // TrackVote
+
 //-----------------------------------------------------------------------------
-void TrackVote::voteTrack(std::string track)
+/** Sets that this vote is for the specified track.
+ */
+void TrackVote::voteTrack(const std::string &track)
 {
     track_info.track = track;
     has_voted_track = true;
-}
+}   // voteTrack
+
 //-----------------------------------------------------------------------------
+/** Sets if this vote is for normal or reversed driving.
+ *  \param reversed True if this vote is for reversed racing.
+ */
 void TrackVote::voteReversed(bool reversed)
 {
     track_info.reversed = reversed;
     has_voted_reversed = true;
-}
+}   // voteReversed
+
 //-----------------------------------------------------------------------------
+/** Votes for the number of laps.
+ *  \param laps Numger of laps.
+ */
 void TrackVote::voteLaps(uint8_t laps)
 {
     track_info.laps = laps;
     has_voted_laps = true;
-}
+}   // voteLaps
 
 //-----------------------------------------------------------------------------
 //---------------------------------  RaceVote ---------------------------------
 //-----------------------------------------------------------------------------
-
+/** Constructor for a race vote.
+ */
 RaceVote::RaceVote()
 {
     m_has_voted_major = false;
@@ -84,115 +102,131 @@ RaceVote::RaceVote()
     m_major_mode = 0;
     m_minor_mode = 0;
     m_races_count = 0;
-}
+    m_tracks_vote.resize(1);
+}   // RaceVote
 
 //-----------------------------------------------------------------------------
-
-void RaceVote::voteMajor(uint8_t major)
+/** Sets the selected major race vote.
+ */
+void RaceVote::voteMajor(uint32_t major)
 {
     m_has_voted_major = true;
-    m_major_mode = major;
-}
+    m_major_mode      = major;
+}   // voteMajor
 
 //-----------------------------------------------------------------------------
-
+/** Sets the vote for race count.
+ */
 void RaceVote::voteRaceCount(uint8_t count)
 {
     m_has_voted_races_count = true;
     m_races_count = count;
-}
+}   // voteRaceCount
 
 //-----------------------------------------------------------------------------
-
-void RaceVote::voteMinor(uint8_t minor)
+/** Sets vote for minor race mode.
+ */
+void RaceVote::voteMinor(uint32_t minor)
 {
     m_has_voted_minor = true;
     m_minor_mode = minor;
-}
+}   // voteMinor
 
 //-----------------------------------------------------------------------------
-
-void RaceVote::voteTrack(std::string track, uint8_t track_number)
+/** Sets a track vote.
+ *  \param track Name of the track.
+ */
+void RaceVote::voteTrack(const std::string &track, uint8_t track_number)
 {
     m_tracks_vote[track_number].voteTrack(track);
-}
+}   // voteTrack
 
 //-----------------------------------------------------------------------------
-
+/** Sets a vote for reveresed racing.
+ */
 void RaceVote::voteReversed(bool reversed, uint8_t track_number)
 {
     m_tracks_vote[track_number].voteReversed(reversed);
-}
+}   // voteReversed
 
 //-----------------------------------------------------------------------------
-
+/** Sets a vote for number of laps.
+ */
 void RaceVote::voteLaps(uint8_t laps, uint8_t track_number)
 {
     m_tracks_vote[track_number].voteLaps(laps);
-}
+}   // voteLaps
 
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedMajor() const
 {
     return m_has_voted_major;
-}
+}   // hasVotedMajor
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedRacesCount() const
 {
     return m_has_voted_races_count;
-}
+}   // hasVotedRacesCount
+
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedMinor() const
 {
     return m_has_voted_minor;
-}
+}   // hasVotedMinor
+
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedTrack(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].has_voted_track;
-}
+}   // hasVotedTrack
+
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedReversed(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].has_voted_reversed;
-}
+}   // hasVotedReversed
+
 //-----------------------------------------------------------------------------
 bool RaceVote::hasVotedLaps(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].has_voted_laps;
-}
+}   // hasVotedLaps
 
 //-----------------------------------------------------------------------------
-
 uint8_t RaceVote::getMajorVote() const
 {
     return m_major_mode;
-}
+}   // getMajorVote
+
 //-----------------------------------------------------------------------------
 uint8_t RaceVote::getRacesCountVote() const
 {
     return m_races_count;
-}
+}   // getRacesCountVote
+
 //-----------------------------------------------------------------------------
 uint8_t RaceVote::getMinorVote() const
 {
     return m_minor_mode;
-}
+}   // getMinorVote
+
 //-----------------------------------------------------------------------------
-std::string RaceVote::getTrackVote(uint8_t track_number) const
+const std::string &RaceVote::getTrackVote(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].track_info.track;
-}
+}   // getTrackVote
+
 //-----------------------------------------------------------------------------
 bool RaceVote::getReversedVote(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].track_info.reversed;
-}
+}   // getReversedVote
+
 //-----------------------------------------------------------------------------
 uint8_t RaceVote::getLapsVote(uint8_t track_number) const
 {
     return m_tracks_vote[track_number].track_info.laps;
-}
+}   // getLapsVote
 
 //-----------------------------------------------------------------------------
 //---------------------------------  RaceConfig -------------------------------
@@ -200,70 +234,65 @@ uint8_t RaceVote::getLapsVote(uint8_t track_number) const
 
 RaceConfig::RaceConfig()
 {
-    m_max_players = 0;
-}
+    m_max_players = NetworkConfig::get()->getMaxPlayers();
+}   // RaceConfig
 
 //-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerCount(uint8_t count)
-{
-    m_max_players = count;
-    m_votes.resize(m_max_players);
-}
-
-//-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerMajorVote(uint8_t player_id, uint8_t major)
+void RaceConfig::setPlayerMajorVote(uint8_t player_id, uint32_t major)
 {
     Log::info("RaceConfig", "Player %d voted for major %d", player_id, major);
     m_votes[player_id].voteMajor(major);
-}
+}   // setPlayerMajorVote
 
 //-----------------------------------------------------------------------------
-
 void RaceConfig::setPlayerRaceCountVote(uint8_t player_id, uint8_t count)
 {
-    Log::info("RaceConfig", "Player %d voted for %d races in GP", player_id, count);
+    Log::info("RaceConfig", "Player %d voted for %d races in GP",
+              player_id, count);
     m_votes[player_id].voteRaceCount(count);
-}
+}   // setPlayerRaceCountVote
 
 //-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerMinorVote(uint8_t player_id, uint8_t minor)
+void RaceConfig::setPlayerMinorVote(uint8_t player_id, uint32_t minor)
 {
     Log::info("RaceConfig", "Player %d voted for minor %d", player_id, minor);
     m_votes[player_id].voteMinor(minor);
-}
+}   // setPlayerMinorVote
 
 //-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerTrackVote(uint8_t player_id, std::string track, uint8_t track_nb)
+void RaceConfig::setPlayerTrackVote(uint8_t player_id, 
+                                    const std::string &track, uint8_t track_nb)
 {
-    Log::info("RaceConfig", "Player %d voted for track %s", player_id, track.c_str());
+    Log::info("RaceConfig", "Player %d voted for track %s",
+              player_id, track.c_str());
     m_votes[player_id].voteTrack(track, track_nb);
-}
+}   // setPlayerTrackVote
 
 //-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerReversedVote(uint8_t player_id, bool reversed, uint8_t track_nb)
+void RaceConfig::setPlayerReversedVote(uint8_t player_id, bool reversed,
+                                       uint8_t track_nb)
 {
     if (reversed)
-        Log::info("RaceConfig", "Player %d voted map %d to be reversed", player_id, track_nb);
+        Log::info("RaceConfig", "Player %d voted map %d to be reversed",
+                  player_id, track_nb);
     else
-        Log::info("RaceConfig", "Player %d voted map %d NOT to be reversed", player_id, track_nb);
+        Log::info("RaceConfig", "Player %d voted map %d NOT to be reversed",
+                  player_id, track_nb);
     m_votes[player_id].voteReversed(reversed, track_nb);
-}
+}   // setPlayerReversedVote
 
 //-----------------------------------------------------------------------------
-
-void RaceConfig::setPlayerLapsVote(uint8_t player_id, uint8_t lap_count, uint8_t track_nb)
+void RaceConfig::setPlayerLapsVote(uint8_t player_id, uint8_t lap_count,
+                                   uint8_t track_nb)
 {
-    Log::info("RaceConfig", "Player %d voted map %d to have %d laps", player_id, track_nb, lap_count);
+    Log::info("RaceConfig", "Player %d voted map %d to have %d laps",
+              player_id, track_nb, lap_count);
     m_votes[player_id].voteLaps(lap_count, track_nb);
-}
+}   // setPlayerLapsVote
 
 //-----------------------------------------------------------------------------
-
+/** Computes the selected race mode.
+ */
 void RaceConfig::computeRaceMode()
 {
     // calculate the race type and number of tracks (in GP mode).
@@ -308,20 +337,27 @@ void RaceConfig::computeRaceMode()
         }
     }
     // now we know :
-    m_major_mode = (!major_histogram.empty() ? getHighestInHistogram<int>(&major_histogram) : 1);
-    m_races_count = (!minor_histogram.empty() ? getHighestInHistogram<int>(&races_count_histogram) : 1);
-    m_minor_mode = (!minor_histogram.empty() ? getHighestInHistogram<int>(&minor_histogram) : 0);
+    m_major_mode  = getHighestInHistogram(&major_histogram,
+                                          (int)RaceManager::MAJOR_MODE_SINGLE);
+    m_minor_mode  = getHighestInHistogram(&minor_histogram,
+                                     (int)RaceManager::MINOR_MODE_NORMAL_RACE);
 
     if (m_major_mode == RaceManager::MAJOR_MODE_GRAND_PRIX)
+    {
+        m_races_count = getHighestInHistogram(&races_count_histogram, 1);
         m_tracks.resize(m_races_count);
+    }
     else
     {
         m_tracks.resize(1);
         m_races_count = 1;
     }
 
-    Log::info("RaceConfig", "Major mode will be %d with %d races. Minor is %d", m_major_mode, m_races_count, m_minor_mode);
-}
+    Log::info("RaceConfig", "Major mode will be %d with %d races. Minor is %d",
+              m_major_mode, m_races_count, m_minor_mode);
+}   // computeRaceMode
+
+// ----------------------------------------------------------------------------
 void RaceConfig::computeNextTrack()
 {
     for (unsigned int j = 0; j < m_races_count; j++)
@@ -344,7 +380,7 @@ void RaceConfig::computeNextTrack()
                     tracks_histogram[m_votes[i].getTrackVote()] = 1;
                 }
             }
-            else if (m_votes[i].hasVotedReversed())
+            if (m_votes[i].hasVotedReversed())
             {
                 try // reversed
                 {
@@ -355,7 +391,7 @@ void RaceConfig::computeNextTrack()
                     reversed_histogram[m_votes[i].getReversedVote()] = 1;
                 }
             }
-            else if (m_votes[i].hasVotedLaps())
+            if (m_votes[i].hasVotedLaps())
             {
                 try // laps
                 {
@@ -368,21 +404,49 @@ void RaceConfig::computeNextTrack()
             }
         }
         // now find the highest votes
-        m_tracks[j].track = getHighestInHistogram<std::string>(&tracks_histogram);
-        m_tracks[j].reversed = getHighestInHistogram<bool>(&reversed_histogram);
-        m_tracks[j].laps = getHighestInHistogram<int>(&laps_histogram);
+        m_tracks[j].track = getHighestInHistogram<std::string>(&tracks_histogram,
+                                                 UserConfigParams::m_last_track);
+        m_tracks[j].reversed = getHighestInHistogram(&reversed_histogram, false);
+        m_tracks[j].laps = getHighestInHistogram<int>(&laps_histogram,
+                                                   UserConfigParams::m_num_laps); 
         if (m_tracks[j].reversed)
-            Log::info("RaceConfig", "Race %d will be on %s with %d laps and reversed", j, m_tracks[j].track.c_str(), m_tracks[j].laps);
+            Log::info("RaceConfig",
+                      "Race %d will be on %s with %d laps and reversed",
+                       j, m_tracks[j].track.c_str(), m_tracks[j].laps);
         else
-            Log::info("RaceConfig", "Race %d will be on %s with %d laps", j, m_tracks[j].track.c_str(), m_tracks[j].laps);
+            Log::info("RaceConfig", "Race %d will be on %s with %d laps", 
+                      j, m_tracks[j].track.c_str(), m_tracks[j].laps);
     }
-}
+}   // computeNextTrack
 
 //-----------------------------------------------------------------------------
+/** Computes the selected setting (based on the users' vote) and sets them
+ *  in the race manager.
+ */
+void RaceConfig::setRaceData()
+{
+    computeRaceMode();
+    computeNextTrack();
+    race_manager->startSingleRace(m_tracks[0].track, m_tracks[0].laps,
+                                  m_tracks[0].reversed);
+}   // setRaceData
 
+//-----------------------------------------------------------------------------
 const TrackInfo* RaceConfig::getNextTrackInfo() const
 {
     return &m_tracks[0];
-}
+}   // getNextTrackInfo
 
 //-----------------------------------------------------------------------------
+int RaceConfig::getNumTrackVotes() const
+{
+    int count = 0;
+
+    for (auto entry : m_votes)
+    {
+        if (entry.second.hasVotedTrack())
+            count++;
+    }
+
+    return count;
+}   // getNumTrackVotes
\ No newline at end of file
