@@ -28,7 +28,7 @@
 #include "items/flyable.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/rescue_animation.hpp"
-#include "karts/controller/player_controller.hpp"
+#include "karts/controller/local_player_controller.hpp"
 #include "modes/soccer_world.hpp"
 #include "modes/world.hpp"
 #include "karts/explosion_animation.hpp"
@@ -218,15 +218,15 @@ void Physics::update(float dt)
             }
             else if (obj->isFlattenKartObject())
             {
-                const KartProperties* kp = kart->getKartProperties();
-                kart->setSquash(kp->getSquashDuration() * kart->getPlayerDifficulty()->getSquashDuration(),
-                    kp->getSquashSlowdown() * kart->getPlayerDifficulty()->getSquashSlowdown());
+                const KartProperties *kp = kart->getKartProperties();
+                kart->setSquash(kp->getSwatterSquashDuration(),
+                    kp->getSwatterSquashSlowdown());
             }
             else if(obj->isSoccerBall() && 
                     race_manager->getMinorMode() == RaceManager::MINOR_MODE_SOCCER)
             {
                 SoccerWorld* soccerWorld = (SoccerWorld*)World::getWorld();
-                soccerWorld->setLastKartTohitBall(kartId);
+                soccerWorld->setBallHitter(kartId);
             }
             continue;
         }
@@ -248,9 +248,9 @@ void Physics::update(float dt)
             else if (anim->isFlattenKartObject())
             {
                 AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
-                const KartProperties* kp = kart->getKartProperties();
-                kart->setSquash(kp->getSquashDuration() * kart->getPlayerDifficulty()->getSquashDuration(),
-                    kp->getSquashSlowdown() * kart->getPlayerDifficulty()->getSquashSlowdown());
+                const KartProperties *kp = kart->getKartProperties();
+                kart->setSquash(kp->getSwatterSquashDuration(),
+                    kp->getSwatterSquashSlowdown());
             }
             continue;
 
@@ -288,7 +288,7 @@ void Physics::update(float dt)
             {
                 int kartId = p->getUserPointer(0)->getPointerFlyable()->getOwnerId();
                 SoccerWorld* soccerWorld = (SoccerWorld*)World::getWorld();
-                soccerWorld->setLastKartTohitBall(kartId);
+                soccerWorld->setBallHitter(kartId);
             }
 
         }
@@ -307,14 +307,14 @@ void Physics::update(float dt)
 
                 // Check for achievements
                 AbstractKart * kart = World::getWorld()->getKart(f->getOwnerId());
-                PlayerController *c = dynamic_cast<PlayerController*>(kart->getController());
+                LocalPlayerController *lpc =
+                    dynamic_cast<LocalPlayerController*>(kart->getController());
 
                 // Check that it's not a kart hitting itself (this can
                 // happen at the time a flyable is shot - release too close
                 // to the kart, and it's the current player. At this stage
                 // only the current player can get achievements.
-                if (target_kart != kart && c &&
-                    c->getPlayer()->getConstProfile() == PlayerManager::getCurrentPlayer())
+                if (target_kart != kart && lpc && lpc->canGetAchievements())
                 {
                     // Compare the current value of hits with the 'hit' goal value
                     // (otherwise it would be compared with the kart id goal value,
@@ -559,15 +559,24 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
             }
             else if(upB->is(UserPointer::UP_PHYSICAL_OBJECT))
             {
-                int n = contact_manifold->getContactPoint(0).m_index1;
-                const Material *m
-                    = n>=0 ? upA->getPointerTriangleMesh()->getMaterial(n)
-                           : NULL;
-                const btVector3 &normal = contact_manifold->getContactPoint(0)
-                                                           .m_normalWorldOnB;
-                upB->getPointerPhysicalObject()->hit(m, normal);
-            }
-        }
+                std::vector<int> used;
+                for(int i=0; i< contact_manifold->getNumContacts(); i++)
+                {
+                    int n = contact_manifold->getContactPoint(i).m_index0;
+                    // Make sure to call the callback function only once
+                    // per triangle.
+                    if(std::find(used.begin(), used.end(), n)!=used.end())
+                        continue;
+                    used.push_back(n);
+                    const Material *m
+                        = n >= 0 ? upB->getPointerTriangleMesh()->getMaterial(n)
+                        : NULL;
+                    const btVector3 &normal = contact_manifold->getContactPoint(i)
+                        .m_normalWorldOnB;
+                    upA->getPointerPhysicalObject()->hit(m, normal);
+                }   // for i in getNumContacts()
+            }   // upB is physical object
+        }   // upA is track
         // 2) object a is a kart
         // =====================
         else if(upA->is(UserPointer::UP_KART))
@@ -646,15 +655,24 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                     upB, contact_manifold->getContactPoint(0).m_localPointB);
             else if(upB->is(UserPointer::UP_TRACK))
             {
-                int n = contact_manifold->getContactPoint(0).m_index1;
-                const Material *m
-                    = n>=0 ? upB->getPointerTriangleMesh()->getMaterial(n)
-                           : NULL;
-                const btVector3 &normal = contact_manifold->getContactPoint(0)
-                                                           .m_normalWorldOnB;
-                upA->getPointerPhysicalObject()->hit(m, normal);
-            }
-        }
+                std::vector<int> used;
+                for(int i=0; i< contact_manifold->getNumContacts(); i++)
+                {
+                    int n = contact_manifold->getContactPoint(i).m_index1;
+                    // Make sure to call the callback function only once
+                    // per triangle.
+                    if(std::find(used.begin(), used.end(), n)!=used.end())
+                        continue;
+                    used.push_back(n);
+                    const Material *m
+                        = n >= 0 ? upB->getPointerTriangleMesh()->getMaterial(n)
+                        : NULL;
+                    const btVector3 &normal = contact_manifold->getContactPoint(i)
+                                             .m_normalWorldOnB;
+                    upA->getPointerPhysicalObject()->hit(m, normal);
+                }   // for i in getNumContacts()
+            }   // upB is track
+        }   // upA is physical object
         else if (upA->is(UserPointer::UP_ANIMATION))
         {
             if(upB->is(UserPointer::UP_KART))
