@@ -26,6 +26,7 @@
 #include "input/input_device.hpp"
 #include "input/input_manager.hpp"
 #include "modes/world.hpp"
+#include "scriptengine/property_animator.hpp"
 #include "states_screens/dialogs/tutorial_message_dialog.hpp"
 #include "states_screens/dialogs/race_paused_dialog.hpp"
 #include "tracks/track.hpp"
@@ -86,7 +87,7 @@ namespace Scripting
 
         void createTextBillboard(std::string* text, SimpleVec3* location)
         {
-            core::stringw wtext = StringUtils::utf8_to_wide(text->c_str());
+            core::stringw wtext = StringUtils::utf8ToWide(*text);
             core::dimension2d<u32> textsize = GUIEngine::getHighresDigitFont()
                 ->getDimension(wtext.c_str());
 
@@ -98,8 +99,8 @@ namespace Scripting
             {
                 gui::ScalableFont* font = GUIEngine::getHighresDigitFont();
                 STKTextBillboard* tb = new STKTextBillboard(wtext.c_str(), font,
-                    video::SColor(255, 255, 225, 0),
-                    video::SColor(255, 255, 89, 0),
+                    GUIEngine::getSkin()->getColor("font::bottom"),
+                    GUIEngine::getSkin()->getColor("font::top"),
                     irr_driver->getSceneManager()->getRootSceneNode(),
                     irr_driver->getSceneManager(), -1, xyz,
                     core::vector3df(1.5f, 1.5f, 1.5f));
@@ -111,14 +112,14 @@ namespace Scripting
                 scene::ISceneManager* sm = irr_driver->getSceneManager();
                 scene::ISceneNode* sn =
                     sm->addBillboardTextSceneNode(GUIEngine::getHighresDigitFont(),
-                    wtext.c_str(),
-                    NULL,
-                    core::dimension2df(textsize.Width / 35.0f,
-                    textsize.Height / 35.0f),
-                    xyz,
-                    -1, // id
-                    video::SColor(255, 255, 225, 0),
-                    video::SColor(255, 255, 89, 0));
+                        wtext.c_str(),
+                        NULL,
+                        core::dimension2df(textsize.Width / 35.0f,
+                            textsize.Height / 35.0f),
+                        xyz,
+                        -1, // id
+                        GUIEngine::getSkin()->getColor("font::bottom"),
+                        GUIEngine::getSkin()->getColor("font::top"));
                 World::getWorld()->getTrack()->addNode(sn);
             }
         }
@@ -133,6 +134,53 @@ namespace Scripting
         {
             new RacePausedDialog(0.8f, 0.6f);
         }
+
+        int getNumberOfKarts()
+        {
+            return race_manager->getNumberOfKarts();
+        }
+
+        int getNumLocalPlayers()
+        {
+            return race_manager->getNumLocalPlayers();
+        }
+
+        bool isTrackReverse()
+        {
+            return race_manager->getReverseTrack();
+        }
+
+        void setFog(float maxDensity, float start, float end, int r, int g, int b, float duration)
+        {
+            PropertyAnimator* animator = PropertyAnimator::get();
+            ::Track* track = World::getWorld()->getTrack();
+            animator->add(
+                new AnimatedProperty(FOG_MAX, 1,
+                    new double[1] { track->getFogMax() }, 
+                    new double[1] { maxDensity }, duration, track)
+            );
+            animator->add(
+                new AnimatedProperty(FOG_RANGE, 2,
+                    new double[2] { track->getFogStart(), track->getFogEnd() },
+                    new double[2] { start, end }, duration, track)
+            );
+
+            video::SColor color = track->getFogColor();
+            animator->add(
+                new AnimatedProperty(FOG_COLOR, 3,
+                    new double[3] { 
+                        (double)color.getRed(), 
+                        (double)color.getGreen(), 
+                        (double)color.getBlue() 
+                    },
+                    new double[3] { 
+                        (double)r,
+                        (double)g,
+                        (double)b
+                    }, 
+                    duration, track)
+            );
+        }
     }
 
     /** \cond DOXYGEN_IGNORE */
@@ -213,6 +261,35 @@ namespace Scripting
             /** @} */
         }
 
+        // ----------- Light Object methods -----------
+
+        namespace Light
+        {
+            /**
+            * @addtogroup Scripting_Light Light (script binding)
+            * Type returned by trackObject.getLight()
+            * @{
+            */
+
+            void setEnergy(float energy, /** \cond DOXYGEN_IGNORE */void *memory /** \endcond */)
+            {
+                ((TrackObjectPresentationLight*)memory)->setEnergy(energy);
+            }
+
+            void animateEnergy(float energy, float duration, /** \cond DOXYGEN_IGNORE */void *memory /** \endcond */)
+            {
+                TrackObjectPresentationLight* light = ((TrackObjectPresentationLight*)memory);
+                PropertyAnimator::get()->add(
+                    new AnimatedProperty(AP_LIGHT_ENERGY, 1,
+                    new double[1] { light->getEnergy() },
+                    new double[1] { energy }, duration, light)
+                );
+            }
+
+            /** @} */
+        }
+
+
         // ----------- Sound Object methods -----------
 
         namespace SoundEmitter
@@ -289,6 +366,7 @@ namespace Scripting
             r = engine->RegisterObjectType("ParticleEmitter", 0, asOBJ_REF | asOBJ_NOCOUNT); assert(r >= 0);
             r = engine->RegisterObjectType("SoundEmitter", 0, asOBJ_REF | asOBJ_NOCOUNT); assert(r >= 0);
             r = engine->RegisterObjectType("Animator", 0, asOBJ_REF | asOBJ_NOCOUNT); assert(r >= 0);
+            r = engine->RegisterObjectType("Light", 0, asOBJ_REF | asOBJ_NOCOUNT); assert(r >= 0);
 
             //r = engine->RegisterGlobalFunction("void disableTrackObject(const string &in)", asFUNCTION(disableTrackObject), asCALL_CDECL); assert(r >= 0);
             //r = engine->RegisterGlobalFunction("void enableTrackObject(const string &in)", asFUNCTION(enableTrackObject), asCALL_CDECL); assert(r >= 0);
@@ -301,10 +379,15 @@ namespace Scripting
             r = engine->RegisterGlobalFunction("TrackObject@ getTrackObject(const string &in, const string &in)", asFUNCTION(getTrackObject), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("void exitRace()", asFUNCTION(exitRace), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("void pauseRace()", asFUNCTION(pauseRace), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("void setFog(float maxDensity, float start, float end, int r, int g, int b, float duration)", asFUNCTION(setFog), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("int getNumberOfKarts()", asFUNCTION(getNumberOfKarts), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("int getNumLocalPlayers()", asFUNCTION(getNumLocalPlayers), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("bool isReverse()", asFUNCTION(isTrackReverse), asCALL_CDECL); assert(r >= 0);
 
             // TrackObject
             r = engine->RegisterObjectMethod("TrackObject", "void setEnabled(bool status)", asMETHOD(::TrackObject, setEnabled), asCALL_THISCALL); assert(r >= 0);
             r = engine->RegisterObjectMethod("TrackObject", "SoundEmitter@ getSoundEmitter()", asMETHOD(::TrackObject, getSoundEmitter), asCALL_THISCALL); assert(r >= 0);
+            r = engine->RegisterObjectMethod("TrackObject", "Light@ getLight()", asMETHOD(::TrackObject, getLight), asCALL_THISCALL); assert(r >= 0);
             r = engine->RegisterObjectMethod("TrackObject", "PhysicalObject@ getPhysics()", asMETHOD(::TrackObject, getPhysics), asCALL_THISCALL); assert(r >= 0);
             r = engine->RegisterObjectMethod("TrackObject", "Mesh@ getMesh()", asMETHOD(::TrackObject, getMesh), asCALL_THISCALL); assert(r >= 0);
             r = engine->RegisterObjectMethod("TrackObject", "ParticleEmitter@ getParticleEmitter()", asMETHOD(::TrackObject, getParticleEmitter), asCALL_THISCALL); assert(r >= 0);
@@ -335,6 +418,9 @@ namespace Scripting
             r = engine->RegisterObjectMethod("SoundEmitter", "void playOnce()", asFUNCTION(SoundEmitter::playOnce), asCALL_CDECL_OBJLAST); assert(r >= 0);
             r = engine->RegisterObjectMethod("SoundEmitter", "void playLoop()", asFUNCTION(SoundEmitter::playLoop), asCALL_CDECL_OBJLAST); assert(r >= 0);
 
+            // Light
+            r = engine->RegisterObjectMethod("Light", "void setEnergy(float)", asFUNCTION(Light::setEnergy), asCALL_CDECL_OBJLAST); assert(r >= 0);
+            r = engine->RegisterObjectMethod("Light", "void animateEnergy(float, float)", asFUNCTION(Light::animateEnergy), asCALL_CDECL_OBJLAST); assert(r >= 0);
 
             // Curve based Animation
             //fails due to insufficient visibility to scripts TODO : Decide whether to fix visibility or introduce wrappers
