@@ -31,6 +31,7 @@
 #include "input/input_manager.hpp"
 #include "input/device_manager.hpp"
 #include "items/item_manager.hpp"
+#include "karts/abstract_characteristic.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/kart_properties_manager.hpp"
 #include "modes/overworld.hpp"
@@ -810,14 +811,19 @@ void KartSelectionScreen::updateKartStats(uint8_t widget_id,
                     kart_properties_manager->getKart(selection);
     if (kp != NULL)
     {
-        w->setValue(KartStatsWidget::SKILL_MASS, (int)(kp->getMass()/5));
-        w->setValue(KartStatsWidget::SKILL_SPEED, (int)((kp->getAbsMaxSpeed()-20)*9));
-        w->setValue(KartStatsWidget::SKILL_POWER, (int)(kp->getAvgPower()));
+        // Scale the values so they look better
+        w->setValue(KartStatsWidget::SKILL_MASS, (int)
+            ((kp->getCombinedCharacteristic()->getMass() - 20) / 4));
+        w->setValue(KartStatsWidget::SKILL_SPEED, (int)
+            ((kp->getCombinedCharacteristic()->getEngineMaxSpeed() - 15) * 6));
+        w->setValue(KartStatsWidget::SKILL_POWER, (int)
+            ((kp->getAvgPower() - 30) / 20));
         w->update(0);
     }
 }
 
-void KartSelectionScreen::updateKartWidgetModel(uint8_t widget_id,
+// ----------------------------------------------------------------------------
+void KartSelectionScreen::updateKartWidgetModel(int widget_id,
                 const std::string& selection,
                 const irr::core::stringw& selectionText)
 {
@@ -920,8 +926,8 @@ void KartSelectionScreen::addMultiplayerMessage()
     {
         m_multiplayer_message = new BubbleWidget();
         m_multiplayer_message->m_properties[PROP_TEXT_ALIGN] = "center";
-        m_multiplayer_message->setText( _("Everyone:\nPress 'Select' now to "
-                                          "join the game!") );
+        m_multiplayer_message->setText( _("Everyone:\nPress the 'Select' button to "
+                                          "join the game") );
         m_multiplayer_message->m_x = message_x;
         m_multiplayer_message->m_y = (int) (fullarea->m_y + fullarea->m_h * 0.3f);
         m_multiplayer_message->m_w = (int) (splitWidth * 0.6f);
@@ -1125,14 +1131,11 @@ void KartSelectionScreen::allPlayersDone()
             ->incrementUseFrequency();
     }
     // ---- Give player info to race manager
-    race_manager->setNumLocalPlayers( players.size() );
+    race_manager->setNumPlayers(players.size());
 
     // ---- Manage 'random kart' selection(s)
     RandomGenerator random;
 
-    //m_kart_widgets.clearAndDeleteAll();
-    //race_manager->setLocalKartInfo(0, w->getSelectionIDString());
-
     std::vector<ItemDescription> items = w->getItems();
 
     // remove the 'random' item itself
@@ -1191,7 +1194,7 @@ void KartSelectionScreen::allPlayersDone()
             }
         }
 
-        race_manager->setLocalKartInfo(n, selected_kart);
+        race_manager->setPlayerKart(n, selected_kart);
 
         // Set per player difficulty if needed
         if (m_multiplayer && UserConfigParams::m_per_player_difficulty &&
@@ -1202,15 +1205,10 @@ void KartSelectionScreen::allPlayersDone()
     // ---- Switch to assign mode
     input_manager->getDeviceManager()->setAssignMode(ASSIGN);
 
-    if (!m_multiplayer)
-    {
-        input_manager->getDeviceManager()
-            ->setSinglePlayer( StateManager::get()->getActivePlayer(0) );
-    }
-    else
-    {
-        input_manager->getDeviceManager()->setSinglePlayer( NULL );
-    }
+    StateManager::ActivePlayer *ap = m_multiplayer 
+                                   ? NULL 
+                                   : StateManager::get()->getActivePlayer(0);
+    input_manager->getDeviceManager()->setSinglePlayer(ap);
 
     // ---- Go to next screen or return to overworld
     if (m_from_overworld || m_go_to_overworld_next)
