@@ -44,7 +44,7 @@
 #include "states_screens/help_screen_1.hpp"
 #include "states_screens/offline_kart_selection.hpp"
 #include "states_screens/online_profile_achievements.hpp"
-#include "states_screens/online_screen.hpp"
+#include "states_screens/online_profile_servers.hpp"
 #include "states_screens/options_screen_video.hpp"
 #include "states_screens/state_manager.hpp"
 #include "states_screens/user_screen.hpp"
@@ -72,8 +72,10 @@ bool MainMenuScreen::m_enable_online = false;
 
 // ----------------------------------------------------------------------------
 
-MainMenuScreen::MainMenuScreen() : Screen("main.stkgui")
+MainMenuScreen::MainMenuScreen() : Screen("main_menu.stkgui")
 {
+    m_online_string = _("Online");
+    m_login_string = _("Login");
 }   // MainMenuScreen
 
 // ----------------------------------------------------------------------------
@@ -167,12 +169,12 @@ void MainMenuScreen::onUpdate(float delta)
     {
         m_user_id->setText(player->getLastOnlineName() + "@stk");
         m_online->setActive(true);
-        m_online->setLabel( _("Online"));
+        m_online->setLabel(m_online_string);
     }
     else if (PlayerManager::getCurrentOnlineState() == PlayerProfile::OS_SIGNED_OUT)
     {
         m_online->setActive(true);
-        m_online->setLabel( _("Login" ));
+        m_online->setLabel(m_login_string);
         m_user_id->setText(player->getName());
     }
     else
@@ -182,8 +184,8 @@ void MainMenuScreen::onUpdate(float delta)
         m_user_id->setText(player->getName());
     }
 
-    m_online->setLabel(PlayerManager::getCurrentOnlineId() ? _("Online")
-                                                           : _("Login" )  );
+    m_online->setLabel(PlayerManager::getCurrentOnlineId() ? m_online_string
+                                                           : m_login_string);
     IconButtonWidget* addons_icon = getWidget<IconButtonWidget>("addons");
     if (addons_icon != NULL)
     {
@@ -245,7 +247,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts( 0 );
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("endcutscene", 999, false);
 
         std::vector<std::string> parts;
@@ -268,7 +270,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts(0);
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("gpwin", 999, false);
         GrandPrixWin* scene = GrandPrixWin::getInstance();
         scene->push();
@@ -281,7 +283,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts(0);
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("gplose", 999, false);
         GrandPrixLose* scene = GrandPrixLose::getInstance();
         scene->push();
@@ -302,7 +304,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts(0);
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("featunlocked", 999, false);
 
         FeatureUnlockedCutScene* scene =
@@ -355,7 +357,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts(0);
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("introcutscene", 999, false);
 
         std::vector<std::string> parts;
@@ -372,7 +374,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
         race_manager->setNumKarts(0);
         race_manager->setNumPlayers(0);
-        race_manager->setNumLocalPlayers(0);
+        race_manager->setNumPlayers(0);
         race_manager->startSingleRace("endcutscene", 999, false);
 
         std::vector<std::string> parts;
@@ -414,7 +416,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
     }
     else if (selection == "startTutorial")
     {
-        race_manager->setNumLocalPlayers(1);
+        race_manager->setNumPlayers(1);
         race_manager->setMajorMode (RaceManager::MAJOR_MODE_SINGLE);
         race_manager->setMinorMode (RaceManager::MINOR_MODE_TUTORIAL);
         race_manager->setNumKarts( 1 );
@@ -435,7 +437,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
                       UserConfigParams::m_default_kart.c_str());
             UserConfigParams::m_default_kart.revertToDefaults();
         }
-        race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart);
+        race_manager->setPlayerKart(0, UserConfigParams::m_default_kart);
 
         // ASSIGN should make sure that only input from assigned devices
         // is read.
@@ -457,7 +459,6 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
             race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
             race_manager->setNumKarts( 0 );
             race_manager->setNumPlayers(0);
-            race_manager->setNumLocalPlayers(0);
             race_manager->startSingleRace("introcutscene", 999, false);
 
             std::vector<std::string> parts;
@@ -491,15 +492,12 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
                                 "\"Connect to the Internet\"."));
             return;
         }
+        
         if (PlayerManager::getCurrentOnlineId())
         {
-            // For 0.8.2 disable the server menu, instead go to online profile
-            #ifdef ENABLE_NETWORK_MULTIPLAYER_SCREEN
-            OnlineScreen::getInstance()->push();
-            #else
             ProfileManager::get()->setVisiting(PlayerManager::getCurrentOnlineId());
+            //OnlineProfileServers::getInstance()->push();
             TabOnlineProfileAchievements::getInstance()->push();
-            #endif
         }
         else
         {
