@@ -22,11 +22,13 @@
 
 #include "audio/music_manager.hpp"
 #include "config/user_config.hpp"
-#include "graphics/camera.hpp"
 #include "graphics/2dutils.hpp"
+#include "graphics/camera.hpp"
+#include "graphics/central_settings.hpp"
 #include "graphics/irr_driver.hpp"
 #include "graphics/material.hpp"
 #include "graphics/material_manager.hpp"
+#include "graphics/post_processing.hpp"
 #include "graphics/referee.hpp"
 #include "guiengine/scalable_font.hpp"
 #include "io/file_manager.hpp"
@@ -57,7 +59,6 @@ namespace irr
 RaceGUIBase::RaceGUIBase()
 {
     m_ignore_unimportant_messages = false;
-    m_lightning             = 0.0f;
     m_max_font_height       = GUIEngine::getFontHeight() + 10;
     m_small_font_max_height = GUIEngine::getSmallFontHeight() + 5;
     //I18N: as in "ready, set, go", shown at the beginning of the race
@@ -296,7 +297,7 @@ void RaceGUIBase::cleanupMessages(const float dt)
 /** Draws the powerup icons on the screen (called once for each player).
  *  \param kart The kart for which to draw the powerup icons.
  *  \param viewport The viewport into which to draw the icons.
- *  \param scaling The scaling to use when draing the icons.
+ *  \param scaling The scaling to use when drawing the icons.
  */
 void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
                                    const core::recti &viewport,
@@ -308,17 +309,26 @@ void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
         || kart->hasFinishedRace()) return;
 
     int n = kart->getPowerup()->getNum() ;
+    int many_powerups = 0;
     if (n<1) return;    // shouldn't happen, but just in case
-    if (n>5) n=5;       // Display at most 5 items
+    if (n>5)
+    {
+        many_powerups = n;
+        n = 1;
+    }
 
-    int nSize = (int)(64.0f*std::min(scaling.X, scaling.Y));
+    float scale = (float)(std::min(scaling.X, scaling.Y));
 
-    int itemSpacing = (int)(std::min(scaling.X, scaling.Y)*30);
+    int nSize = (int)(64.0f * scale);
+
+    int itemSpacing = (int)(scale * 30);
 
     int x1 = viewport.UpperLeftCorner.X  + viewport.getWidth()/2
            - (n * itemSpacing)/2;
     int y1 = viewport.UpperLeftCorner.Y  + (int)(20 * scaling.Y);
 
+    int x2 = 0;
+
     assert(powerup != NULL);
     assert(powerup->getIcon() != NULL);
     video::ITexture *t=powerup->getIcon()->getTexture();
@@ -327,11 +337,21 @@ void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
 
     for ( int i = 0 ; i < n ; i++ )
     {
-        int x2 = (int)(x1+i*itemSpacing);
+        x2 = (int)(x1+i*itemSpacing);
         core::rect<s32> pos(x2, y1, x2+nSize, y1+nSize);
         draw2DImage(t, pos, rect, NULL,
                                                   NULL, true);
     }   // for i
+
+    if (many_powerups > 0)
+    {
+        gui::ScalableFont* font = GUIEngine::getHighresDigitFont();
+        core::rect<s32> pos(x2+nSize, y1, x2+nSize+nSize, y1+nSize);
+        font->setScale(scale);
+        font->draw(core::stringw(L"x")+StringUtils::toWString(many_powerups),
+            pos, video::SColor(255, 255, 255, 255));
+        font->setScale(1.0f);
+    }
 }   // drawPowerupIcons
 
 // ----------------------------------------------------------------------------
@@ -339,7 +359,6 @@ void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
  */
 void RaceGUIBase::renderGlobal(float dt)
 {
-    if (m_lightning > 0.0f) m_lightning -= dt;
 
 }   // renderGlobal
 
@@ -410,62 +429,7 @@ void RaceGUIBase::preRenderCallback(const Camera *camera)
 // ----------------------------------------------------------------------------
 void RaceGUIBase::renderPlayerView(const Camera *camera, float dt)
 {
-#if 0
-    const core::recti &viewport = camera->getViewport();
 
-    if (m_lightning > 0.0f)
-    {
-        GLint glviewport[4];
-        glviewport[0] = viewport.UpperLeftCorner.X;
-        glviewport[1] = viewport.UpperLeftCorner.Y;
-        glviewport[2] = viewport.LowerRightCorner.X;
-        glviewport[3] = viewport.LowerRightCorner.Y;
-        //glGetIntegerv(GL_VIEWPORT, glviewport);
-
-        if (!irr::video::useCoreContext)
-            glDisable(GL_TEXTURE_2D);
-        glDisable(GL_DEPTH_TEST);
-        glDisable(GL_BLEND);
-        glEnable(GL_BLEND);
-        glBlendFunc(GL_ONE, GL_ONE);
-        glColor4f(0.7f*m_lightning, 0.7f*m_lightning, 0.7f*std::min(1.0f, m_lightning*1.5f), 1.0f);
-        glEnable(GL_COLOR_MATERIAL);
-        glDisable(GL_CULL_FACE);
-        glBegin(GL_QUADS);
-
-        glVertex3d(glviewport[0],glviewport[1],0);
-        glVertex3d(glviewport[0],glviewport[3],0);
-        glVertex3d(glviewport[2],glviewport[3],0);
-        glVertex3d(glviewport[2],glviewport[1],0);
-        glEnd();
-        if (!irr::video::useCoreContext)
-            glEnable(GL_TEXTURE_2D);
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-    }
-    else
-    {
-        GLint glviewport[4];
-        glGetIntegerv(GL_VIEWPORT, glviewport);
-
-        glDisable(GL_TEXTURE_2D);
-        glDisable(GL_DEPTH_TEST);
-        glDisable(GL_BLEND);
-        glEnable(GL_BLEND);
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-        glColor4f(0.0f, 0.0f, 0.0f, 0.4f);
-
-        glEnable(GL_COLOR_MATERIAL);
-        glDisable(GL_CULL_FACE);
-        glBegin(GL_QUADS);
-
-        glVertex3d(glviewport[0],glviewport[1],0);
-        glVertex3d(glviewport[0],glviewport[3],0);
-        glVertex3d(glviewport[2],glviewport[3],0);
-        glVertex3d(glviewport[2],glviewport[1],0);
-        glEnd();
-        glEnable(GL_BLEND);
-    }
-#endif
 }   // renderPlayerView
 
 
@@ -494,11 +458,15 @@ void RaceGUIBase::drawGlobalMusicDescription()
     gui::IGUIFont*       font = GUIEngine::getFont();
 
     float race_time = World::getWorld()->getTime();
-    // In follow the leader the clock counts backwards, so convert the
+    // In the modes that the clock counts backwards, convert the
     // countdown time to time since start:
     if(race_manager->getMinorMode()==RaceManager::MINOR_MODE_FOLLOW_LEADER)
         race_time = ((FollowTheLeaderRace*)World::getWorld())->getClockStartTime()
                   - race_time;
+    else if (race_manager->getMinorMode()==RaceManager::MINOR_MODE_SOCCER &&
+             race_manager->hasTimeTarget())
+        race_time = race_manager->getTimeTarget() - World::getWorld()->getTime();
+
     // ---- Manage pulsing effect
     // 3.0 is the duration of ready/set (TODO: don't hardcode)
     float timeProgression = (float)(race_time) /
@@ -836,13 +804,13 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
         // draw icon
         video::ITexture *icon =
         kart->getKartProperties()->getIconMaterial()->getTexture();
-        int w =
-        kart->getController()->isPlayerController() ? ICON_PLAYER_WIDTH
-        : ICON_WIDTH;
+        int w = kart->getController()
+                    ->isLocalPlayerController() ? ICON_PLAYER_WIDTH
+                                                : ICON_WIDTH;
         const core::rect<s32> pos(x, y, x+w, y+w);
 
         //to bring to light the player's icon: add a background
-        if (kart->getController()->isPlayerController())
+        if (kart->getController()->isLocalPlayerController())
         {
             video::SColor colors[4];
             for (unsigned int i=0;i<4;i++)
@@ -862,8 +830,7 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
         {
             const core::rect<s32> rect(core::position2d<s32>(0,0),
                                        icon->getSize());
-            draw2DImage(icon, pos, rect,
-                                                      NULL, NULL, true);
+            draw2DImage(icon, pos, rect, NULL, NULL, true, kart->isGhostKart());
         }
 
         //draw status info - icon fade out in case of rescue/explode
