@@ -54,6 +54,7 @@
 #include "physics/triangle_mesh.hpp"
 #include "race/race_manager.hpp"
 #include "tracks/bezier_curve.hpp"
+#include "tracks/battle_graph.hpp"
 #include "tracks/check_manager.hpp"
 #include "tracks/model_definition_loader.hpp"
 #include "tracks/track_manager.hpp"
@@ -81,7 +82,8 @@
 using namespace irr;
 
 
-const float Track::NOHIT           = -99999.9f;
+const float Track::NOHIT               = -99999.9f;
+bool        Track::m_dont_load_navmesh = false;
 
 // ----------------------------------------------------------------------------
 Track::Track(const std::string &filename)
@@ -114,7 +116,9 @@ Track::Track(const std::string &filename)
     m_enable_push_back      = true;
     m_reverse_available     = false;
     m_is_arena              = false;
+    m_max_arena_players     = 0;
     m_has_easter_eggs       = false;
+    m_has_navmesh           = false;
     m_is_soccer             = false;
     m_is_cutscene           = false;
     m_camera_far            = 1000.0f;
@@ -274,6 +278,7 @@ void Track::reset()
 void Track::cleanup()
 {
     QuadGraph::destroy();
+    BattleGraph::destroy();
     ItemManager::destroy();
     VAOManager::kill();
 
@@ -489,6 +494,7 @@ void Track::loadTrackInfo()
 
     if(!root || root->getName()!="track")
     {
+        delete root;
         std::ostringstream o;
         o<<"Can't load track '"<<m_filename<<"', no track element.";
         throw std::runtime_error(o.str());
@@ -507,6 +513,7 @@ void Track::loadTrackInfo()
     root->get("gravity",               &m_gravity);
     root->get("soccer",                &m_is_soccer);
     root->get("arena",                 &m_is_arena);
+    root->get("max-arena-players",     &m_max_arena_players);
     root->get("cutscene",              &m_is_cutscene);
     root->get("groups",                &m_groups);
     root->get("internal",              &m_internal);
@@ -581,6 +588,20 @@ void Track::loadTrackInfo()
         }
         delete easter;
     }
+
+    if(file_manager->fileExists(m_root+"navmesh.xml") && !m_dont_load_navmesh)
+        m_has_navmesh = true;
+    else if ( (m_is_arena || m_is_soccer) && !m_dont_load_navmesh)
+    {
+        Log::warn("Track", "NavMesh is not found for arena %s, "
+                  "disable AI for it.\n", m_name.c_str());
+    }
+    if (m_is_soccer)
+    {
+        // Currently only max eight players in soccer mode
+        m_max_arena_players = 8;
+    }
+
 }   // loadTrackInfo
 
 //-----------------------------------------------------------------------------
@@ -645,6 +666,25 @@ void Track::startMusic() const
         music_manager->clearCurrentMusic();
 }   // startMusic
 
+//-----------------------------------------------------------------------------
+/** Loads the polygon graph for battle, i.e. the definition of all polys, and the way
+ *  they are connected to each other. Input file name is hardcoded for now
+ */
+void Track::loadBattleGraph(const XMLNode &node)
+{
+    BattleGraph::create(m_root+"navmesh.xml", node);
+
+    if(BattleGraph::get()->getNumNodes()==0)
+    {
+        Log::warn("track", "No graph nodes defined for track '%s'\n",
+                m_filename.c_str());
+    }
+    else
+    {
+        loadMinimap();
+    }
+}   // loadBattleGraph
+
 //-----------------------------------------------------------------------------
 /** Loads the quad graph, i.e. the definition of all quads, and the way
  *  they are connected to each other.
@@ -675,42 +715,18 @@ void Track::loadQuadGraph(unsigned int mode_id, const bool reverse)
     }
     else
     {
-        //Check whether the hardware can do nonsquare or
-        // non power-of-two textures
-        video::IVideoDriver* const video_driver = irr_driver->getVideoDriver();
-        bool nonpower = false; //video_driver->queryFeature(video::EVDF_TEXTURE_NPOT);
-        bool nonsquare =
-            video_driver->queryFeature(video::EVDF_TEXTURE_NSQUARE);
-
-        //Create the minimap resizing it as necessary.
-        m_mini_map_size = World::getWorld()->getRaceGUI()->getMiniMapSize();
-        core::dimension2du size = m_mini_map_size
-                                 .getOptimalSize(!nonpower,!nonsquare);
-
-        QuadGraph::get()->makeMiniMap(size, "minimap::" + m_ident, video::SColor(127, 255, 255, 255),
-            &m_old_rtt_mini_map, &m_new_rtt_mini_map);
-        if (m_old_rtt_mini_map)
-        {
-            m_minimap_x_scale = float(m_mini_map_size.Width) / float(m_old_rtt_mini_map->getSize().Width);
-            m_minimap_y_scale = float(m_mini_map_size.Height) / float(m_old_rtt_mini_map->getSize().Height);
-        }
-        else if (m_new_rtt_mini_map)
-        {
-            m_minimap_x_scale = float(m_mini_map_size.Width) / float(m_new_rtt_mini_map->getWidth());
-            m_minimap_y_scale = float(m_mini_map_size.Height) / float(m_new_rtt_mini_map->getHeight());
-        }
-        else
-        {
-            m_minimap_x_scale = 0;
-            m_minimap_y_scale = 0;
-        }
+        loadMinimap();
     }
 }   // loadQuadGraph
 
 // -----------------------------------------------------------------------------
+
 void Track::mapPoint2MiniMap(const Vec3 &xyz, Vec3 *draw_at) const
 {
-    QuadGraph::get()->mapPoint2MiniMap(xyz, draw_at);
+    if ((m_is_arena || m_is_soccer) && m_has_navmesh)
+        BattleGraph::get()->mapPoint2MiniMap(xyz, draw_at);
+    else
+        QuadGraph::get()->mapPoint2MiniMap(xyz, draw_at);
     draw_at->setX(draw_at->getX() * m_minimap_x_scale);
     draw_at->setY(draw_at->getY() * m_minimap_y_scale);
 }
@@ -915,18 +931,6 @@ void Track::convertTrackToBullet(scene::ISceneNode *node)
         if (mb->getVertexType() == video::EVT_STANDARD)
         {
             irr::video::S3DVertex* mbVertices=(video::S3DVertex*)mb->getVertices();
-            if (race_manager->getReverseTrack() &&
-                material->getMirrorAxisInReverse() != ' ')
-            {
-                for (unsigned int i = 0; i < mb->getVertexCount(); i++)
-                {
-                    core::vector2df &tc = mb->getTCoords(i);
-                    if (material->getMirrorAxisInReverse() == 'V')
-                        tc.Y = 1 - tc.Y;
-                    else
-                        tc.X = 1 - tc.X;
-                }
-            }   // reverse track and texture needs mirroring
             for (unsigned int matrix_index = 0; matrix_index < matrices.size(); matrix_index++)
             {
                 for (unsigned int j = 0; j < mb->getIndexCount(); j += 3)
@@ -1007,6 +1011,50 @@ void Track::convertTrackToBullet(scene::ISceneNode *node)
 
 }   // convertTrackToBullet
 
+// ----------------------------------------------------------------------------
+
+void Track::loadMinimap()
+{
+    //Check whether the hardware can do nonsquare or
+    // non power-of-two textures
+    video::IVideoDriver* const video_driver = irr_driver->getVideoDriver();
+    bool nonpower = false; //video_driver->queryFeature(video::EVDF_TEXTURE_NPOT);
+    bool nonsquare =
+        video_driver->queryFeature(video::EVDF_TEXTURE_NSQUARE);
+
+    //Create the minimap resizing it as necessary.
+    m_mini_map_size = World::getWorld()->getRaceGUI()->getMiniMapSize();
+    core::dimension2du size = m_mini_map_size
+                             .getOptimalSize(!nonpower,!nonsquare);
+
+    if ((m_is_arena || m_is_soccer) && m_has_navmesh)
+    {
+        BattleGraph::get()->makeMiniMap(size, "minimap::" + m_ident, video::SColor(127, 255, 255, 255),
+            &m_old_rtt_mini_map, &m_new_rtt_mini_map);
+    }
+    else
+    {
+        QuadGraph::get()->makeMiniMap(size, "minimap::" + m_ident, video::SColor(127, 255, 255, 255),
+            &m_old_rtt_mini_map, &m_new_rtt_mini_map);
+    }
+
+    if (m_old_rtt_mini_map)
+    {
+        m_minimap_x_scale = float(m_mini_map_size.Width) / float(m_old_rtt_mini_map->getSize().Width);
+        m_minimap_y_scale = float(m_mini_map_size.Height) / float(m_old_rtt_mini_map->getSize().Height);
+    }
+    else if (m_new_rtt_mini_map)
+    {
+        m_minimap_x_scale = float(m_mini_map_size.Width) / float(m_new_rtt_mini_map->getWidth());
+        m_minimap_y_scale = float(m_mini_map_size.Height) / float(m_new_rtt_mini_map->getHeight());
+    }
+    else
+    {
+        m_minimap_x_scale = 0;
+        m_minimap_y_scale = 0;
+    }
+}   // loadMinimap
+
 // ----------------------------------------------------------------------------
 /** Loads the main track model (i.e. all other objects contained in the
  *  scene might use raycast on this track model to determine the actual
@@ -1345,6 +1393,7 @@ void Track::handleAnimatedTextures(scene::ISceneNode *node, const XMLNode &xml)
         // to lower case, for case-insensitive comparison
         name = StringUtils::toLowerCase(name);
 
+        int moving_textures_found = 0;
         for(unsigned int i=0; i<node->getMaterialCount(); i++)
         {
             video::SMaterial &irrMaterial=node->getMaterial(i);
@@ -1361,8 +1410,12 @@ void Track::handleAnimatedTextures(scene::ISceneNode *node, const XMLNode &xml)
                 if (texture_name != name) continue;
                 core::matrix4 *m = &irrMaterial.getTextureMatrix(j);
                 m_animated_textures.push_back(new MovingTexture(m, *texture_node));
+                moving_textures_found++;
             }   // for j<MATERIAL_MAX_TEXTURES
         }   // for i<getMaterialCount
+
+        if (moving_textures_found == 0)
+            Log::warn("AnimTexture", "Did not find animate texture '%s'", name.c_str());
     }   // for node_number < xml->getNumNodes
 }   // handleAnimatedTextures
 
@@ -1582,23 +1635,9 @@ void Track::loadTrackModel(bool reverse_track, unsigned int mode_id)
         (void)e;
     }
 
-    // Load the graph only now: this function is called from world, after
-    // the race gui was created. The race gui is needed since it stores
-    // the information about the size of the texture to render the mini
-    // map to.
-    if (!m_is_arena && !m_is_soccer && !m_is_cutscene) loadQuadGraph(mode_id, reverse_track);
-
-    ItemManager::create();
-
-    // Set the default start positions. Node that later the default
-    // positions can still be overwritten.
-    float forwards_distance  = 1.5f;
-    float sidewards_distance = 3.0f;
-    float upwards_distance   = 0.1f;
-    int   karts_per_row      = 2;
-
-
     // Start building the scene graph
+    // Soccer field with navmesh requires it
+    // for two goal line to be drawn them in minimap
     std::string path = m_root + m_all_modes[mode_id].m_scene;
     XMLNode *root    = file_manager->createXMLTree(path);
 
@@ -1612,6 +1651,23 @@ void Track::loadTrackModel(bool reverse_track, unsigned int mode_id)
         throw std::runtime_error(msg.str());
     }
 
+    // Load the graph only now: this function is called from world, after
+    // the race gui was created. The race gui is needed since it stores
+    // the information about the size of the texture to render the mini
+    // map to.
+    if (!m_is_arena && !m_is_soccer && !m_is_cutscene) loadQuadGraph(mode_id, reverse_track);
+    else if ((m_is_arena || m_is_soccer) && !m_is_cutscene && m_has_navmesh)
+        loadBattleGraph(*root);
+
+    ItemManager::create();
+
+    // Set the default start positions. Node that later the default
+    // positions can still be overwritten.
+    float forwards_distance  = 1.5f;
+    float sidewards_distance = 3.0f;
+    float upwards_distance   = 0.1f;
+    int   karts_per_row      = 2;
+
     const XMLNode *default_start = root->getNode("default-start");
     if (default_start)
     {
@@ -1695,7 +1751,7 @@ void Track::loadTrackModel(bool reverse_track, unsigned int mode_id)
     // It's important to execute this BEFORE the code that creates the skycube,
     // otherwise the skycube node could be modified to have fog enabled, which
     // we don't want
-    if (m_use_fog && !UserConfigParams::m_camera_debug && !CVS->isGLSL())
+    if (m_use_fog && !Camera::isDebug() && !CVS->isGLSL())
     {
         /* NOTE: if LINEAR type, density does not matter, if EXP or EXP2, start
            and end do not matter */
@@ -1796,21 +1852,29 @@ void Track::loadTrackModel(bool reverse_track, unsigned int mode_id)
 
     createPhysicsModel(main_track_count);
 
+    const bool arena_random_item_created =
+        ItemManager::get()->randomItemsForArena(m_start_transforms);
 
-    for (unsigned int i=0; i<root->getNumNodes(); i++)
+    if (!arena_random_item_created)
     {
-        const XMLNode *node = root->getNode(i);
-        const std::string &name = node->getName();
-        if (name=="banana"      || name=="item"      ||
-            name=="small-nitro" || name=="big-nitro" ||
-            name=="easter-egg"                           )
+        for (unsigned int i=0; i<root->getNumNodes(); i++)
         {
-            itemCommand(node);
-        }
-    }   // for i<root->getNumNodes()
+            const XMLNode *node = root->getNode(i);
+            const std::string &name = node->getName();
+            if (name=="banana"      || name=="item"      ||
+                name=="small-nitro" || name=="big-nitro" ||
+                name=="easter-egg"                           )
+            {
+                itemCommand(node);
+            }
+        }   // for i<root->getNumNodes()
+    }
 
     delete root;
 
+    if ((m_is_arena || m_is_soccer) && !m_is_cutscene && m_has_navmesh && !arena_random_item_created)
+        BattleGraph::get()->findItemsOnGraphNodes();
+
     if (UserConfigParams::m_track_debug &&
         race_manager->getMinorMode()!=RaceManager::MINOR_MODE_3_STRIKES &&
         !m_is_cutscene)
@@ -1818,6 +1882,11 @@ void Track::loadTrackModel(bool reverse_track, unsigned int mode_id)
         QuadGraph::get()->createDebugMesh();
     }
 
+    if (UserConfigParams::m_track_debug && m_has_navmesh &&
+        race_manager->getMinorMode()==RaceManager::MINOR_MODE_3_STRIKES &&
+        !m_is_cutscene)
+        BattleGraph::get()->createDebugMesh();
+
     // Only print warning if not in battle mode, since battle tracks don't have
     // any quads or check lines.
     if (CheckManager::get()->getCheckStructureCount()==0  &&
@@ -2349,4 +2418,3 @@ bool Track::findGround(AbstractKart *kart)
 
     return true;
 }   // findGround
-
