@@ -25,6 +25,7 @@
 #include "graphics/camera.hpp"
 #include "graphics/central_settings.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/light.hpp"
 #include "graphics/material_manager.hpp"
 #include "graphics/mesh_tools.hpp"
 #include "graphics/particle_emitter.hpp"
@@ -491,6 +492,10 @@ void TrackObjectPresentationMesh::init(const XMLNode* xml_node,
         m_frame_end = node->getEndFrame();
         if (xml_node)
             xml_node->get("frame-end", &m_frame_end);
+
+        if (World::getWorld() && World::getWorld()->getTrack() && xml_node)
+            World::getWorld()->getTrack()
+            ->handleAnimatedTextures(m_node, *xml_node);
     }
     else
     {
@@ -972,7 +977,16 @@ TrackObjectPresentationLight::TrackObjectPresentationLight(
 TrackObjectPresentationLight::~TrackObjectPresentationLight()
 {
 }   // ~TrackObjectPresentationLight
-
+// ----------------------------------------------------------------------------
+void TrackObjectPresentationLight::setEnergy(float energy)
+{
+    m_energy = energy;
+    LightNode* lnode = dynamic_cast<LightNode*>(m_node);
+    if (lnode != NULL)
+    {
+        lnode->setEnergy(energy);
+    }
+}
 // ----------------------------------------------------------------------------
 TrackObjectPresentationActionTrigger::TrackObjectPresentationActionTrigger(
                                                        const XMLNode& xml_node)
