@@ -354,11 +354,12 @@ private:
     video::SColor m_color;
     float m_distance;
     float m_energy;
-
 public:
     TrackObjectPresentationLight(const XMLNode& xml_node,
                                  scene::ISceneNode* parent);
     virtual ~TrackObjectPresentationLight();
+    float getEnergy() const { return m_energy; }
+    void setEnergy(float energy);
 };   // TrackObjectPresentationLight
 
 // ============================================================================
