@@ -56,6 +56,7 @@ static const char* GPU_Phase[Q_LAST] =
     "Bloom",
     "Tonemap",
     "Motion Blur",
+    "Lightning",
     "MLAA",
     "GUI",
 };
@@ -74,6 +75,7 @@ Profiler profiler;
 
 // --- Begin portable precise timer ---
 #ifdef WIN32
+    #define WIN32_LEAN_AND_MEAN
     #include <windows.h>
 
     double getTimeMilliseconds()
